--[[
	Cooldown Timeline, Vreenak (US-Remulos)
	https://www.curseforge.com/wow/addons/cooldown-timeline
	
	Update notes:
		19a1:
			- Fixed an issue that would cause an error, but not affect the running of the mode
			- Fixed an iusse that caused spells such as Shadowmeld to get stuck and not cooldown correctly
			- Spells will now periodically check if the CD is still active, to allow abilities like Cold Snap, and Premeditation to clear icons
			- Cooldowns with a duration longer than the ignore threshold (long) value will now appear on the timeline at the 100% mark when it drops below the threshold
			- Previously it would appear at the position of its percentage of the total duration and cause longer cooldowns to linger for long periods of time
			- This applies only to the percentage based timeline modes
			- You can now customise the Split (Time) mode to suit your needs (timeline modes now have their own options section)
]]--

CooldownTimeline = LibStub("AceAddon-3.0"):NewAddon("CooldownTimeline", "AceConsole-3.0", "AceEvent-3.0")
CooldownTimeline.Masque = LibStub("Masque", true)
CooldownTimeline.SharedMedia = LibStub( "LibSharedMedia-3.0" )
CooldownTimeline.inCombat = false
CooldownTimeline.ignoreNextManaTick = true
CooldownTimeline.uniqueID = 1000
CooldownTimeline.spellTable = {}
CooldownTimeline.iconTable = {}
CooldownTimeline.characterFilterList = {}
CooldownTimeline.characterFilterListItems = {}
CooldownTimeline.characterFilterListPet = {}
CooldownTimeline.characterHighlightList = {}
CooldownTimeline.characterHighlightItems = {}
CooldownTimeline.characterHighlightPet = {}
CooldownTimeline.stringImportExport = ""

local version = "2.5.1-19a1"
local Masque = CooldownTimeline.Masque
local SharedMedia = CooldownTimeline.SharedMedia

local options = {
	name = "Cooldown Timeline",
	handler = CooldownTimeline,
	type = "group",
	args = {
		--generalOptionsHeader = {
		--	name = "General Options",
		--	type = "header",
		--	order = 1,
		--},
		unlockFrames = {
			name = "Unlock Frames (allow drag and drop movement)",
			desc = "Allows drag and drop movement of the frames",
			order = 1,
			type = "toggle",
			get = function(info) return CooldownTimeline.db.profile.unlockFrames end,
			set = function(info, val) CooldownTimeline.db.profile.unlockFrames = val end,
			width = "full",
		},
		--[[enableGCD = {
			name = "Show the GCD on the timeline as a status bar",
			--desc = "Enables / disables showing the GCD on the Timeline frame",
			type = "toggle",
			order = 2,
			get = function(info) return CooldownTimeline.db.profile.showGCD end,
			set = function(info,val) CooldownTimeline.db.profile.showGCD = val end,
			width = "full",
		},]]--
		hideOutsideCombat = {
			name = "Hide out of combat - they will still show if there is something cooling down",
			--desc = "Will hide the Timeline and Ready frames out of combat unless there is still something cooling down, or 'ready'",
			type = "toggle",
			order = 3.1,
			get = function(info) return CooldownTimeline.db.profile.hideOutsideCombat end,
			set = function(info,val)
					CooldownTimeline.db.profile.hideOutsideCombat = val

					if val then
						CooldownTimeline.fReady:SetAlpha(0)
						CooldownTimeline.fTimeline:SetAlpha(0)
					else
						CooldownTimeline.fReady:SetAlpha(1)
						CooldownTimeline.fTimeline:SetAlpha(1)
					end
					
				end,
			width = "full",
		},
		onlyShowWhenCoolingDown = {
			name = "Only show when cooling down, even in combat ",
			--desc = "Will hide the Timeline and Ready frames out of combat unless there is still something cooling down, or 'ready'",
			type = "toggle",
			order = 3.2,
			get = function(info) return CooldownTimeline.db.profile.onlyShowWhenCoolingDown end,
			set = function(info,val)
					CooldownTimeline.db.profile.onlyShowWhenCoolingDown = val

					if val then
						CooldownTimeline.fReady:SetAlpha(0)
						CooldownTimeline.fTimeline:SetAlpha(0)
					else
						CooldownTimeline.fReady:SetAlpha(1)
						CooldownTimeline.fTimeline:SetAlpha(1)
					end
					
				end,
			width = "full",
		},
		trackItemCooldowns = {
			name = "Track item cooldowns",
			desc = "Will enable tracking cooldowns of items both equipped and in your bags.  *WORK IN PROGRESS* Currently may show lots of things unintentionally",
			type = "toggle",
			hidden = false,
			order = 4,
			get = function(info) return CooldownTimeline.db.profile.trackItemCooldowns end,
			set = function(info,val)
					CooldownTimeline.db.profile.trackItemCooldowns = val
					--CooldownTimeline:ScanInventory()
				end,
			width = "full",
		},
		trackPetSpells = {
			name = "Track pet spells",
			desc = "Track spells cast by your pet",
			type = "toggle",
			hidden = false,
			order = 5,
			get = function(info) return CooldownTimeline.db.profile.trackPetSpells end,
			set = function(info,val)
					CooldownTimeline.db.profile.trackPetSpells = val
					--CooldownTimeline:ScanPetSpellbook()
				end,
			width = "full",
		},
		longIgnoreThreshold = {
			name = "Ignore Threshold (long)",
			desc = "Any cooldowns longer than this value will be hidden",
			order = 6,
			type = "range",
			softMin = 0,
			softMax = 3600,
			get = function(info) return CooldownTimeline.db.profile.longIgnoreThreshold end,
			set = function(info, val) CooldownTimeline.db.profile.longIgnoreThreshold = val end,
		},
		--[[spacer1 = {
			name = "",
			type = "description",
			order = 7,
		},
		optionsExport = {
			name = "Export",
			desc = "",
			order = 7.1,
			type = "execute",
			func = function(info)
					--CooldownTimeline.stringImportExport = "ksdjflkasdj flksadjglksdjf lksadjf lksajglksjadg l"
					local tempString = ""
					
					for key, value in ipairs(CooldownTimeline.db.profile) do
						
						tempString = tempString..key..":"..value..","
					end
					
					CooldownTimeline.stringImportExport = tempString
				end
		},
		optionsImport = {
			name = "Import",
			desc = "",
			order = 7.2,
			type = "execute",
			confirm = true,
			func = function(info)
					
				end
		},
		spacer2 = {
			name = "",
			type = "description",
			order = 8,
		},
		optionsImportExportString = {
			name = "Import",
			desc = "",
			order = 8.1,
			type = "input",
			multiline = 10,
			get = function(info) return CooldownTimeline.stringImportExport end,
			set = function(info,val)
					CooldownTimeline.stringImportExport = val
				end,
			width = "full",
		},]]--
		spacer3 = {
			name = "",
			type = "description",
			order = -1,
		},
		debugFrame = {
			name = "Enable Debug",
			desc = "You do not need this as it adds no functionality, its purely for author tinkering (REQUIRES RELOAD)",
			type = "toggle",
			hidden = false,
			order = -1,
			confirm = true,
			get = function(info) return CooldownTimeline.db.profile.debugFrame end,
			set = function(info,val)
					CooldownTimeline.db.profile.debugFrame = val
					ReloadUI()
				end,
			width = "full",
		},
	}
}

local optionsIcons = {
	name = "Icons",
	handler = CooldownTimeline,
	type = "group",
	args = {
		optionsGeneral = {
			name = "General",
			type = "group",
			args = {
				--[[fIconHeader = {
					name = "Icon Options",
					type = "header",
					order = 1,
				},]]--
				fIconSize = {
					name = "Icon Size",
					desc = "Sets the size of the icon on the timeline",
					order = 1,
					type = "range",
					softMin = 0,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fIconSize end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconSize = val
							for _, child in ipairs(CooldownTimeline.iconTable) do
								if Masque then
									-- Kill masque for this icon(button)
									CooldownTimeline.masqueGroup:RemoveButton(child)
									
									-- Change the settings
									child:SetSize(val, val)
									child:SetPoint("CENTER",0,0)
									child.tex:SetAllPoints(child)	
									
									-- Reapply masque
									CooldownTimeline.masqueGroup = Masque:Group("CooldownTimeline")
									CooldownTimeline.masqueGroup:AddButton(child, { Icon = child.tex })	
								else
									child:SetSize(val, val)
									child:SetPoint("CENTER",0,0)
								end
							end
						end,
				},
				spacer2 = {
					name = "",
					type = "description",
					order = 2.1,
				},
				fIconReadyText = {
					name = "Icon Text",
					desc = "Defines what text is shown on an icon when ready",
					type = "input",
					order = 2.2,
					get = function(info) return CooldownTimeline.db.profile.fIconReadyText end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconReadyText = val
						end,
				},
				fIconFontColor = {
					name = "Color",
					desc = "Sets the font color",
					order = 2.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineFontColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineFontColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineFontColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineFontColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fIconFont = { r = red, g = green, b = blue, a = alpha }
							for _, child in ipairs(CooldownTimeline.iconTable) do
								child.text:SetTextColor(red, green, blue, alpha)
							end
						end,
				},
				spacer3 = {
					name = "",
					type = "description",
					order = 3.1,
				},
				fIconFont = {
					name = "Font",
					desc = "Selects the font used on the icons",
					order = 3.2,
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fIconFont end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconFont = val
							for _, child in ipairs(CooldownTimeline.iconTable) do
								child.text:SetFont(SharedMedia:Fetch("font", val), CooldownTimeline.db.profile.fIconFontSize, "OUTLINE")
							end
						end,
				},
				fIconFontSize = {
					name = "Font Size",
					desc = "Sets the size of the font on the timeline and icons",
					order = 3.3,
					type = "range",
					softMin = 0,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fIconFontSize end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconFontSize = val
							for _, child in ipairs(CooldownTimeline.iconTable) do
								child.text:SetFont(SharedMedia:Fetch("font", CooldownTimeline.db.profile.fIconFont), val, "OUTLINE")
							end
						end,
				},
				spacer4 = {
					name = "",
					type = "description",
					order = 4,
				},
				fIconTextOffset = {
					name = "Text Offset",
					desc = "Sets the vertical offest of the text",
					order = 4.1,
					type = "range",
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fIconTextOffset end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconTextOffset = val
							for _, child in ipairs(CooldownTimeline.iconTable) do
								--child.text:SetFont(SharedMedia:Fetch("font", CooldownTimeline.db.profile.fIconFont), val, "OUTLINE")
								child.text:SetPoint("CENTER", 0, val)
							end
						end,
				},
				spacer5 = {
					name = "\n\n\n",
					type = "description",
					order = 5.1,
				},
				fIconBorder = {
					name = "                    Border Texture",
					desc = "Selects the texture",
					order = 5.2,
					type = "select",
					dialogControl = 'LSM30_Border',
					values = AceGUIWidgetLSMlists.border,
					get = function(info) return CooldownTimeline.db.profile.fIconBorder end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconBorder = val
							CooldownTimeline:RefreshIcons()				
						end,
				},
				fIconBorderColor = {
					name = "Border Color",
					desc = "Selects the border color",
					order = 5.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fIconBorderColor["r"]
							local g = CooldownTimeline.db.profile.fIconBorderColor["g"]
							local b = CooldownTimeline.db.profile.fIconBorderColor["b"]
							local a = CooldownTimeline.db.profile.fIconBorderColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fIconBorderColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline:RefreshIcons()
						end,
				},
				spacer6 = {
					name = "",
					type = "description",
					order = 6.1,
				},
				fIconBorderSize = {
					name = "Border Size",
					desc = "Sets the size of the border",
					order = 6.2,
					type = "range",
					softMin = 1,
					softMax = 40,
					get = function(info) return CooldownTimeline.db.profile.fIconBorderSize end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconBorderSize = val
							CooldownTimeline:RefreshIcons()
						end,
				},
				fIconBorderPadding = {
					name = "Border Padding",
					desc = "Sets the size of the border",
					order = 6.3,
					type = "range",
					softMin = 0,
					softMax = 40,
					get = function(info) return CooldownTimeline.db.profile.fIconBorderPadding end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconBorderPadding = val
							CooldownTimeline:RefreshIcons()
						end,
				},
				spacer7 = {
					name = "\n\n\n",
					type = "description",
					order = 7.1,
				},
				fTimelineIconOffset = {
					name = "Icon Offset",
					desc = "Sets the vertical offset of the icons on the timeline",
					order = 7.2,
					type = "range",
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineIconOffset end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineIconOffset = val
							CooldownTimeline:RefreshIcons()
						end,
				},
				spacer8 = {
					name = "\n\n\n",
					type = "description",
					order = 8.1,
				},
				fIconReadySound = {
					name = "Ready Sound",
					desc = "Selects a sound to play when an icon is 'ready'",
					order = 8.2,
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = AceGUIWidgetLSMlists.sound,
					get = function(info) return CooldownTimeline.db.profile.fIconReadySound end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconReadySound = val
							--CooldownTimeline:RefreshIcons()				
						end,
				},
				spacer9 = {
					name = "\n\n\n",
					type = "description",
					order = 9.1,
				},
				fTimelineStack = {
					name = "Icon Stacking",
					desc = "When enabled icons will offset up/down to avoid overlapping",
					type = "toggle",
					order = 9.2,
					get = function(info) return CooldownTimeline.db.profile.fTimelineStack end,
					set = function(info,val) CooldownTimeline.db.profile.fTimelineStack = val end,
					width = 'full',
				},
				fTimelineStackOverlap = {
					name = "Overlap only Stacking",
					desc = "If enabled, stacking will only occur on overlapped icons",
					type = "toggle",
					order = 9.3,
					get = function(info) return CooldownTimeline.db.profile.fTimelineStackOverlap end,
					set = function(info,val) CooldownTimeline.db.profile.fTimelineStackOverlap = val end,
					width = 'full',
				},
				fTimelineStackMaxSize = {
					name = "Max stack height",
					desc = "Sets how tall the stack will be total (the stack will center vertically on the timeline)",
					order = 9.4,
					type = "range",
					softMin = 0,
					softMax = 100,
					get = function(info) return CooldownTimeline.db.profile.fTimelineStackMaxSize end,
					set = function(info, val) CooldownTimeline.db.profile.fTimelineStackMaxSize = val end,
				},
			}
		},
		optionsHighlight = {
			name = "Highlight",
			type = "group",
			args = {
				--[[fIconHeader = {
					name = "Icon Options",
					type = "header",
					order = 1,
				},]]--
				fIconHighlightEffect = {
					name = "Highlight Effect",
					desc = "This will change what effect is used to highlight icons on the timeline/ready frames",
					order = 1.1,
					type = "select",
					values = {
							["NONE"] = "None",
							["GLOW"] = "Glow",
							["PULSE"] = "Pulse",
							["BORDER"] = "Border",
							["BORDER_PULSE"] = "Border Pulse",
							--["SHAKE"] = "Shake",
							["BOUNCE"] = "Bounce",
							--["SCALE"] = "Scale",
						},
					get = function(info) return CooldownTimeline.db.profile.fIconHighlightEffect end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconHighlightEffect = val
							
							for _, icon in ipairs(CooldownTimeline.iconTable) do
								CooldownTimeline:StopAllHighlights(icon)
							end
						end,
				},
				spacer2 = {
					name = "",
					type = "description",
					order = 2.1,
				},
				fReadyIconHighlightDuration = {
					name = "Icon Duration",
					desc = "Sets the duration a highlighted icon will display as ready",
					order = 2.2,
					type = "range",
					softMin = 0,
					softMax = 20,
					get = function(info) return CooldownTimeline.db.profile.fReadyIconHighlightDuration end,
					set = function(info, val)
							CooldownTimeline.db.profile.fReadyIconHighlightDuration = val
						end,
				},
				fIconHighlightPin = {
					name = "Pin to Ready",
					desc = "This will pin highlighted icons to the ready area until the spell/item is used again",
					type = "toggle",
					order = 2.3,
					get = function(info) return CooldownTimeline.db.profile.fIconHighlightPin end,
					set = function(info,val) CooldownTimeline.db.profile.fIconHighlightPin = val end,
				},
				spacer3 = {
					name = "",
					type = "description",
					order = 3.1,
				},
				fIconHighlightBorder = {
					name = "                    Border Texture",
					desc = "Selects the texture",
					order = 3.2,
					type = "select",
					dialogControl = 'LSM30_Border',
					values = AceGUIWidgetLSMlists.border,
					get = function(info) return CooldownTimeline.db.profile.fIconHighlightBorder end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconHighlightBorder = val
							CooldownTimeline:RefreshIcons()				
						end,
				},
				fIconHighlightBorderColor = {
					name = "Color",
					desc = "Selects the border color",
					order = 3.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fIconHighlightBorderColor["r"]
							local g = CooldownTimeline.db.profile.fIconHighlightBorderColor["g"]
							local b = CooldownTimeline.db.profile.fIconHighlightBorderColor["b"]
							local a = CooldownTimeline.db.profile.fIconHighlightBorderColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fIconHighlightBorderColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline:RefreshIcons()
						end,
				},
				spacer4 = {
					name = "",
					type = "description",
					order = 4.1,
				},
				fIconHighlightBorderSize = {
					name = "Size",
					desc = "Sets the size of the border",
					order = 4.2,
					type = "range",
					softMin = 1,
					softMax = 40,
					get = function(info) return CooldownTimeline.db.profile.fIconHighlightBorderSize end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconHighlightBorderSize = val
							CooldownTimeline:RefreshIcons()
						end,
				},
				fIconHighlightBorderPadding = {
					name = "Padding",
					desc = "Sets the size of the border",
					order = 4.3,
					type = "range",
					softMin = 0,
					softMax = 40,
					get = function(info) return CooldownTimeline.db.profile.fIconHighlightBorderPadding end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconHighlightBorderPadding = val
							CooldownTimeline:RefreshIcons()
						end,
				},
				spacer5 = {
					name = "\n\n\n",
					type = "description",
					order = 5.1,
				},
				fIconReadyHighlightSound = {
					name = "Ready Sound",
					desc = "Selects a sound to play when a highlighted icon is 'ready' (overrides any existing sound for a non-highlighted icon)",
					order = 5.2,
					type = "select",
					dialogControl = 'LSM30_Sound',
					values = AceGUIWidgetLSMlists.sound,
					get = function(info) return CooldownTimeline.db.profile.fIconReadyHighlightSound end,
					set = function(info, val)
							CooldownTimeline.db.profile.fIconReadyHighlightSound = val
							--CooldownTimeline:RefreshIcons()				
						end,
				},
			}
		},
	}
}
	
local optionsTimeline = {
	name = "Timeline",
	handler = CooldownTimeline,
	type = 'group',
	width = 'full',
	args = {
		optionsMode = {
			name = "Mode",
			type = "group",
			args = {
				--[[spacer1 = {
					name = "\n\n\n",
					type = "description",
					order = 1.1,
				},]]--
				fTimelineMode = {
					name = "Timeline Mode",
					desc = "This will change how an icon moves along the timeline",
					order = 1.2,
					type = "select",
					values = { ["LINEAR"] = "Linear (%)", ["SPLIT"] = "Split", ["LINEAR_ABS"] = "Linear (Time)" },
					get = function(info) return CooldownTimeline.db.profile.fTimelineMode end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineMode = val
							--CooldownTimeline:SetTimelineText(true)
							
							CooldownTimeline:UpdateTimelineText(true)
							CooldownTimeline:DrawTimelineText()
						end,
				},
				fTimelineIconReverseDirection = {
					name = "Reverse Travel Direction",
					desc = "Changes the icon travel direction from 'Right->Left' to 'Left->Right'",
					type = "toggle",
					order = 1.3,
					set = function(info,val) CooldownTimeline.db.profile.fTimelineIconReverseDirection = val end,
					get = function(info) return CooldownTimeline.db.profile.fTimelineIconReverseDirection end,
				},
				spacer2 = {
					name = "\n\n\n",
					type = "description",
					order = 2.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "LINEAR_ABS" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineModeAbsLimit = {
					name = "Linear (Time) Max",
					desc = "Sets the max time (in seconds) that the bar will show",
					order = 2.2,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "LINEAR_ABS" then
								return false
							else
								return true
							end
						end,
					type = "range",
					softMin = 0,
					softMax = 600,
					bigStep = 10,
					get = function(info) return CooldownTimeline.db.profile.fTimelineModeAbsLimit end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineModeAbsLimit = val
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer3 = {
					name = "\n\n\n",
					type = "description",
					order = 3.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineModeSplitCount = {
					name = "Split Count",
					desc = "Sets how many splits are in the timeline",
					order = 3.2,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								return false
							else
								return true
							end
						end,
					type = "range",
					min = 1,
					max = 3,
					bigStep = 1,
					get = function(info) return CooldownTimeline.db.profile.fTimelineModeSplitCount end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineModeSplitCount = val
							CooldownTimeline:UpdateTimelineText(true)
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer31 = {
					name = "\n\n\n",
					type = "description",
					order = 31.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineModeSplit1 = {
					name = "Split 1",
					desc = "Sets the time at which the first split occurs",
					order = 31.2,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								return false
							else
								return true
							end
						end,
					type = "range",
					softMin = 0,
					softMax = 600,
					bigStep = 10,
					get = function(info) return CooldownTimeline.db.profile.fTimelineModeSplit1 end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineModeSplit1 = val
							CooldownTimeline:UpdateTimelineText(true)
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer32 = {
					name = "",
					type = "description",
					order = 32.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								if CooldownTimeline.db.profile.fTimelineModeSplitCount >= 2 then
									return false
								end
							end
							return true
						end,
				},
				fTimelineModeSplit2 = {
					name = "Split 2",
					desc = "Sets the time at which the second split occurs",
					order = 32.2,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								if CooldownTimeline.db.profile.fTimelineModeSplitCount >= 2 then
									return false
								end
							end
							return true
						end,
					type = "range",
					softMin = 0,
					softMax = 600,
					bigStep = 10,
					get = function(info) return CooldownTimeline.db.profile.fTimelineModeSplit2 end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineModeSplit2 = val
							CooldownTimeline:UpdateTimelineText(true)
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer33 = {
					name = "",
					type = "description",
					order = 33.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								if CooldownTimeline.db.profile.fTimelineModeSplitCount >= 3 then
									return false
								end
							end
							return true
						end,
				},
				fTimelineModeSplit3 = {
					name = "Split 3",
					desc = "Sets the time at which the first split occurs",
					order = 33.2,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								if CooldownTimeline.db.profile.fTimelineModeSplitCount >= 3 then
									return false
								end
							end
							return true
						end,
					type = "range",
					softMin = 0,
					softMax = 600,
					bigStep = 10,
					get = function(info) return CooldownTimeline.db.profile.fTimelineModeSplit3 end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineModeSplit3 = val
							CooldownTimeline:UpdateTimelineText(true)
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer61 = {
					name = "\n\n\n",
					type = "description",
					order = 61.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineModeSplitLimit = {
					name = "Split Max",
					desc = "Sets the max time (in seconds) that the bar will show",
					order = 61.2,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineMode == "SPLIT" then
								return false
							else
								return true
							end
						end,
					type = "range",
					softMin = 0,
					softMax = 600,
					bigStep = 10,
					get = function(info) return CooldownTimeline.db.profile.fTimelineModeSplitLimit end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineModeSplitLimit = val
							CooldownTimeline:UpdateTimelineText(true)
							CooldownTimeline:SetTimelineText()
						end,
				},
			}
		},
		optionsTimeline = {
			name = "General",
			type = "group",
			args = {
				--[[spacer1 = {
					name = "\n\n\n",
					type = "description",
					order = 1.1,
				},		
				spacer2 = {
					name = "\n\n\n",
					type = "description",
					order = 2.1,
				},]]--
				fTimelineWidth = {
					name = "Width",
					desc = "Sets the width for the Timeline frame",
					order = 2.2,
					type = "range",
					softMin = 0,
					softMax = 600,
					get = function(info) return CooldownTimeline.db.profile.fTimelineWidth end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineWidth = val
							CooldownTimeline.fTimeline:SetWidth(val)
	 				end,
				},
				fTimelineHeight = {
					name = "Height",
					desc = "Sets the height for the Timeline frame",
					order = 2.3,
					type = "range",
					softMin = 0,
					softMax = 600,
					get = function(info) return CooldownTimeline.db.profile.fTimelineHeight end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineHeight = val
							CooldownTimeline.fTimeline:SetHeight(val)
						end,
				},
				spacer3 = {
					name = "",
					type = "description",
					order = 3.1,
				},
				fTimelineX = {
					name = "x Pos",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 3.2,
					type = "range",
					softMin = -500,
					softMax = 500,
					get = function(info) return CooldownTimeline.db.profile.fTimelinePosX end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelinePosX = val
							CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, val, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				fTimelineY = {
					name = "y Pos",
					desc = "Sets the y co-rd for the Timeline frame",
					order = 3.3,
					type = "range",
					softMin = -500,
					softMax = 500,
					get = function(info) return CooldownTimeline.db.profile.fTimelinePosY end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelinePosY = val
							CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, CooldownTimeline.db.profile.fTimelinePosX, val)
						end,
				},
				spacer4 = {
					name = "",
					type = "description",
					order = 4.1,
				},
				fTimelineRelativeTo = {
					name = "Anchor Point",
					desc = "X/Y position is relative to this point of the screen",
					order = 4.2,
					type = "select",
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineRelativeTo end,
					set = function(info, val)
							CooldownTimeline.fTimeline:ClearAllPoints()
							
							CooldownTimeline.db.profile.fTimelineRelativeTo = val
							
							CooldownTimeline.fTimeline:SetPoint(val, CooldownTimeline.db.profile.fTimelinePosX, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				spacer5 = {
					name = "\n\n\n",
					type = "description",
					order = 5.1,
				},
				fTimelineBackground = {
					name = "Background Texture",
					desc = "Selects the texture",
					order = 5.2,
					type = "select",
					dialogControl = 'LSM30_Statusbar',
					values = AceGUIWidgetLSMlists.statusbar,
					get = function(info) return CooldownTimeline.db.profile.fTimelineBackground end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineBackground = val
							CooldownTimeline.fTimeline.bg:SetTexture(SharedMedia:Fetch("statusbar", CooldownTimeline.db.profile.fTimelineBackground))

							local r = CooldownTimeline.db.profile.fTimelineBackgroundColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineBackgroundColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineBackgroundColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineBackgroundColor["a"]
							CooldownTimeline.fTimeline.bg:SetVertexColor(r, g, b, a)
						end,
				},
				fTimelineBackgroundColor = {
					name = "Color",
					desc = "Selects the background color",
					order = 5.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineBackgroundColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineBackgroundColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineBackgroundColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineBackgroundColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineBackgroundColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline.fTimeline.bg:SetVertexColor(red, green, blue, alpha)
						end,
				},
				spacer6 = {
					name = "",
					type = "description",
					order = 6.1,
				},
				fTimelineTexture = {
					name = "Foreground Texture",
					desc = "Selects the texture",
					order = 6.2,
					type = "select",
					dialogControl = 'LSM30_Statusbar',
					values = AceGUIWidgetLSMlists.statusbar,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTexture end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTexture = val
							CooldownTimeline.fTimeline:SetStatusBarTexture(SharedMedia:Fetch("statusbar", val))

							local r = CooldownTimeline.db.profile.fTimelineTextureColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextureColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextureColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextureColor["a"]
							CooldownTimeline.fTimeline:SetStatusBarColor(r, g, b, a)
						end,
				},
				fTimelineTextureColor = {
					name = "Color",
					desc = "Selects the background color",
					order = 6.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTextureColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextureColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextureColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextureColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTextureColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline.fTimeline:SetStatusBarColor(red, green, blue, alpha)
						end,
				},
				spacer7 = {
					name = "\n\n\n",
					type = "description",
					order = 7.1,
				},
				fTimelineFont = {
					name = "Font",
					desc = "Selects the font for the timeline and icons",
					order = 7.2,
					type = "select",
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fTimelineFont end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineFont = val
							CooldownTimeline:SetTimelineText()
						end,
				},
				fTimelineFontColor = {
					name = "Color",
					desc = "Selects the font color",
					order = 7.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineFontColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineFontColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineFontColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineFontColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineFontColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer8 = {
					name = "",
					type = "description",
					order = 8.1,
				},
				fTimelineFontSize = {
					name = "Font Size",
					desc = "Sets the size of the font on the timeline and icons",
					order = 8.2,
					type = "range",
					softMin = 0,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fTimelineFontSize end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineFontSize = val
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer9 = {
					name = "",
					type = "description",
					order = 9.1,
				},
				fTimelineTextOffset = {
					name = "Font Offset",
					desc = "Sets the vertical offset of the text",
					order = 9.2,
					type = "range",
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextOffset end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextOffset = val
							CooldownTimeline:SetTimelineText()
						end,
				},
				spacer10 = {
					name = "\n\n\n",
					type = "description",
					order = 10.1,
				},
				fTimelineBorder = {
					name = "                    Border Texture",
					desc = "Selects the texture",
					order = 10.2,
					type = "select",
					dialogControl = 'LSM30_Border',
					values = AceGUIWidgetLSMlists.border,
					get = function(info) return CooldownTimeline.db.profile.fTimelineBorder end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineBorder = val

							local fTimelineBorderSize = CooldownTimeline.db.profile.fTimelineBorderSize
							local fTimelineBorderInset = CooldownTimeline.db.profile.fTimelineBorderInset
							local fTimelineBorderPadding = CooldownTimeline.db.profile.fTimelineBorderPadding
							local fTimelineBorderColor = CooldownTimeline.db.profile.fTimelineBorderColor
							
							CooldownTimeline:SetBorder(CooldownTimeline.fTimeline, val, fTimelineBorderSize, fTimelineBorderInset)
							CooldownTimeline:SetBorderColor(CooldownTimeline.fTimeline, fTimelineBorderColor)
							CooldownTimeline:SetBorderPoint(CooldownTimeline.fTimeline, fTimelineBorderPadding)
						end,
				},
				fTimelineBorderColor = {
					name = "Color",
					desc = "Selects the border color",
					order = 10.3,
					type = "color",
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineBorderColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineBorderColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineBorderColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineBorderColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineBorderColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline.fTimeline.border:SetBackdropBorderColor(red, green, blue, alpha)
						end,
				},
				spacer11 = {
					name = "",
					type = "description",
					order = 11.1,
				},
				fTimelineBorderSize = {
					name = "Size",
					desc = "Sets the size of the border",
					order = 11.2,
					type = "range",
					softMin = 1,
					softMax = 40,
					get = function(info) return CooldownTimeline.db.profile.fTimelineBorderSize end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineBorderSize = val
							local fTimelineBorder = CooldownTimeline.db.profile.fTimelineBorder
							local fTimelineBorderInset = CooldownTimeline.db.profile.fTimelineBorderInset
							local fTimelineBorderColor = CooldownTimeline.db.profile.fTimelineBorderColor
							
							CooldownTimeline:SetBorder(CooldownTimeline.fTimeline, fTimelineBorder, val, fTimelineBorderInset)
							CooldownTimeline:SetBorderColor(CooldownTimeline.fTimeline, fTimelineBorderColor)
						end,
				},
				fTimelineBorderPadding = {
					name = "Padding",
					desc = "Sets the size of the border",
					order = 11.3,
					type = "range",
					softMin = 0,
					softMax = 40,
					get = function(info) return CooldownTimeline.db.profile.fTimelineBorderPadding end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineBorderPadding = val
							local fTimelineBorderColor = CooldownTimeline.db.profile.fTimelineBorderColor
							
							CooldownTimeline:SetBorderPoint(CooldownTimeline.fTimeline, val)
							CooldownTimeline:SetBorderColor(CooldownTimeline.fTimeline, fTimelineBorderColor)
						end,
				},
				spacer12 = {
					name = "\n\n\n",
					type = "description",
					order = 12.1,
				},
				fTimelineAnimateInType = {
					name = "IN Animation",
					desc = "Select the animation played on show",
					order = 12.2,
					type = "select",
					values = { ["FADE"] = "Fade" },
					get = function(info) return CooldownTimeline.db.profile.fTimelineAnimateInType end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineAnimateInType = val
						end,
				},
				fTimelineAnimateIn = {
					name = "Enabled",
					desc = "Enable/disable animated on show",
					type = "toggle",
					order = 12.3,
					set = function(info,val) CooldownTimeline.db.profile.fTimelineAnimateIn = val end,
					get = function(info) return CooldownTimeline.db.profile.fTimelineAnimateIn end,
				},
				spacer13 = {
					name = "",
					type = "description",
					order = 13.1,
				},
				fTimelineAnimateOutType = {
					name = "OUT Animation",
					desc = "Select the animation played on hide",
					order = 13.2,
					type = "select",
					values = { ["FADE"] = "Fade" },
					get = function(info) return CooldownTimeline.db.profile.fTimelineAnimateOutType end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineAnimateOutType = val
						end,
				},
				fTimelineAnimateOut = {
					name = "Enabled",
					desc = "Enable/disable animated on hide",
					type = "toggle",
					order = 13.3,
					set = function(info,val) CooldownTimeline.db.profile.fTimelineAnimateOut = val end,
					get = function(info) return CooldownTimeline.db.profile.fTimelineAnimateOut end,
				},
			}
		},
		optionsText = {
			name = "Text",
			type = "group",
			args = {
				fTimelineTextHeader = {
					name = "Default Text",
					type = "header",
					order = 1.0,
				},
				--[[spacer1 = {
					name = "",
					type = "description",
					order = 1.1,
				},]]--
				fTimelineText1Text = {
					name = "",
					type = "input",
					order = 1.2,
					disabled = true,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[1] == nil
							return not CooldownTimeline.db.profile.fTimelineText[1]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[1] then
								return CooldownTimeline.db.profile.fTimelineText[1]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[1] then
								CooldownTimeline.db.profile.fTimelineText[1]["text"] = val
							end
						end,
				},
				fTimelineText1Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 1.4,
					type = "toggle",
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[1] == nil
							return not CooldownTimeline.db.profile.fTimelineText[1]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[1] then
								return CooldownTimeline.db.profile.fTimelineText[1]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[1] then
								CooldownTimeline.db.profile.fTimelineText[1]["enabled"] = val
								CooldownTimeline:DrawTimelineText()
							end
						end,
				},
				spacer2 = {
					name = "",
					type = "description",
					order = 2.1,
				},
				fTimelineText2Text = {
					name = "",
					type = "input",
					order = 2.2,
					disabled = true,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[2] == nil
							return not CooldownTimeline.db.profile.fTimelineText[2]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[2] then
								return CooldownTimeline.db.profile.fTimelineText[2]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[2] then
								CooldownTimeline.db.profile.fTimelineText[2]["text"] = val
							end
						end,
				},
				fTimelineText2Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 2.3,
					type = "toggle",
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[2] == nil
							return not CooldownTimeline.db.profile.fTimelineText[2]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[2] then
								return CooldownTimeline.db.profile.fTimelineText[2]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[2] then
								CooldownTimeline.db.profile.fTimelineText[2]["enabled"] = val
								CooldownTimeline:DrawTimelineText()
							end
						end,
				},
				spacer3 = {
					name = "",
					type = "description",
					order = 3.1,
				},
				fTimelineText3Text = {
					name = "",
					type = "input",
					order = 3.2,
					disabled = true,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[3] == nil
							return not CooldownTimeline.db.profile.fTimelineText[3]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[3] then
								return CooldownTimeline.db.profile.fTimelineText[3]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[3] then
								CooldownTimeline.db.profile.fTimelineText[3]["text"] = val
							end
						end,
				},
				fTimelineText3Enabled = {
					name = "Enabled",
					desc = "Show this text",
					type = "toggle",
					order = 3.3,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[3] == nil
							return not CooldownTimeline.db.profile.fTimelineText[3]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[3] then
								return CooldownTimeline.db.profile.fTimelineText[3]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[3] then
								CooldownTimeline.db.profile.fTimelineText[3]["enabled"] = val
								CooldownTimeline:DrawTimelineText()
							end
						end,
				},
				spacer4 = {
					name = "",
					type = "description",
					order = 4.1,
				},
				fTimelineText4Text = {
					name = "",
					type = "input",
					order = 4.2,
					disabled = true,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[4] == nil
							return not CooldownTimeline.db.profile.fTimelineText[4]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[4] then
								return CooldownTimeline.db.profile.fTimelineText[4]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[4] then
								CooldownTimeline.db.profile.fTimelineText[4]["text"] = val
							end
						end,
				},
				fTimelineText4Enabled = {
					name = "Enabled",
					desc = "Show this text",
					type = "toggle",
					order = 4.3,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[4] == nil
							return not CooldownTimeline.db.profile.fTimelineText[4]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[4] then
								return CooldownTimeline.db.profile.fTimelineText[4]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[4] then
								CooldownTimeline.db.profile.fTimelineText[4]["enabled"] = val
								CooldownTimeline:DrawTimelineText()
							end
						end,
				},
				spacer5 = {
					name = "",
					type = "description",
					order = 5.1,
				},
				fTimelineText5Text = {
					name = "",
					type = "input",
					order = 5.2,
					disabled = true,
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[5] == nil
							return not CooldownTimeline.db.profile.fTimelineText[5]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[5] then
								return CooldownTimeline.db.profile.fTimelineText[5]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[5] then
								CooldownTimeline.db.profile.fTimelineText[5]["text"] = val
							end
						end,
				},
				fTimelineText5Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 5.3,
					type = "toggle",
					hidden = function(info)
							--return CooldownTimeline.db.profile.fTimelineText[5] == nil
							return not CooldownTimeline.db.profile.fTimelineText[5]["used"]
						end,
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineText[5] then
								return CooldownTimeline.db.profile.fTimelineText[5]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineText[5] then
								CooldownTimeline.db.profile.fTimelineText[5]["enabled"] = val
								CooldownTimeline:DrawTimelineText()
							end
						end,
				},
				fTimelineTextCustomHeader = {
					name = "Custom Text",
					type = "header",
					order = 6.0,
				},
				spacer6 = {
					name = "\nYou can use custom text tags to display 'useful' dynamic information.\nAll tags currently supported can be viewed at the following link:\n\nhttps://www.curseforge.com/wow/addons/cooldown-timeline/pages/custom-text-tags\n\n",
					type = "description",
					order = 6.01,
				},
				fTimelineText6Text = {
					name = "Custom Text 1",
					desc = function(info) return CooldownTimeline.GetCustomTagDescription() end,
					type = "input",
					disabled = function(info) return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					order = 6.02,
					width = "double",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[1] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[1]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[1] then
								CooldownTimeline.db.profile.fTimelineTextCustom[1]["text"] = val
							end
						end,
				},
				fTimelineText6Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 6.03,
					type = "toggle",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[1] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[1] then
								CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] = val
							end
						end,
				},
				fTimelineText6Color = {
					name = "Color",
					desc = "Selects the text color",
					order = 6.04,
					type = "color",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTextCustom[1]["color"]["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextCustom[1]["color"]["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextCustom[1]["color"]["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextCustom[1]["color"]["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["color"] = { r = red, g = green, b = blue, a = alpha }
						end,
				},
				fTimelineText6Font = {
					name = "Font",
					desc = "Selects the font for the timeline and icons",
					order = 6.05,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[1]["font"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["font"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				fTimelineText6Size = {
					name = "Size",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 6.06,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					softMin = 1,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[1]["size"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["size"] = val
						end,
				},
				spacer61 = {
					name = "",
					type = "description",
					order = 6.07,
				},
				fTimelineText6XOffset = {
					name = "x Pos",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 6.08,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					softMin = -100,
					softMax = 100,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[1]["xOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["xOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, val, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				fTimelineText6YOffset = {
					name = "y Pos",
					desc = "Sets the y co-rd for the Timeline frame",
					order = 6.09,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[1]["yOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["yOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, CooldownTimeline.db.profile.fTimelinePosX, val)
						end,
				},
				--[[fTimelineText6Align = {
					name = "Align",
					desc = "X/Y position is relative to this point of the screen",
					order = 6.10,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[1]["align"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["align"] = val
							CooldownTimeline:SetTimelineText()
						end,
				},]]--
				fTimelineText6Anchor = {
					name = "Anchor Point",
					desc = "X/Y position is relative to this point of the screen",
					order = 6.11,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[1]["anchor"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[1]["anchor"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				spacer62 = {
					name = "\n\n",
					type = "description",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[1]["enabled"] end,
					order = 6.12,
				},
								spacer7 = {
					name = "",
					type = "description",
					order = 7.01,
				},
				fTimelineText7Text = {
					name = "Custom Text 2",
					desc = function(info) return CooldownTimeline.GetCustomTagDescription() end,
					type = "input",
					disabled = function(info) return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					order = 7.02,
					width = "double",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[2] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[2]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[2] then
								CooldownTimeline.db.profile.fTimelineTextCustom[2]["text"] = val
							end
						end,
				},
				fTimelineText7Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 7.03,
					type = "toggle",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[2] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[2] then
								CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] = val
							end
						end,
				},
				fTimelineText7Color = {
					name = "Color",
					desc = "Selects the text color",
					order = 7.04,
					type = "color",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTextCustom[2]["color"]["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextCustom[2]["color"]["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextCustom[2]["color"]["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextCustom[2]["color"]["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["color"] = { r = red, g = green, b = blue, a = alpha }
						end,
				},
				fTimelineText7Font = {
					name = "Font",
					desc = "Selects the font for the timeline and icons",
					order = 7.05,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[2]["font"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["font"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				fTimelineText7Size = {
					name = "Size",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 7.06,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					softMin = 1,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[2]["size"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["size"] = val
						end,
				},
				spacer71 = {
					name = "",
					type = "description",
					order = 7.07,
				},
				fTimelineText7XOffset = {
					name = "x Pos",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 7.08,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					softMin = -100,
					softMax = 100,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[2]["xOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["xOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, val, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				fTimelineText7YOffset = {
					name = "y Pos",
					desc = "Sets the y co-rd for the Timeline frame",
					order = 7.09,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[2]["yOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["yOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, CooldownTimeline.db.profile.fTimelinePosX, val)
						end,
				},
				--[[fTimelineText7Align = {
					name = "Align",
					desc = "X/Y position is relative to this point of the screen",
					order = 7.10,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[2]["align"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["align"] = val
							CooldownTimeline:SetTimelineText()
						end,
				},]]--
				fTimelineText7Anchor = {
					name = "Anchor Point",
					desc = "X/Y position is relative to this point of the screen",
					order = 7.11,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[2]["anchor"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[2]["anchor"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				spacer72 = {
					name = "\n\n",
					type = "description",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[2]["enabled"] end,
					order = 7.12,
				},
								spacer8 = {
					name = "",
					type = "description",
					order = 8.01,
				},
				fTimelineText8Text = {
					name = "Custom Text 3",
					desc = function(info) return CooldownTimeline.GetCustomTagDescription() end,
					type = "input",
					disabled = function(info) return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					order = 8.02,
					width = "double",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[3] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[3]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[3] then
								CooldownTimeline.db.profile.fTimelineTextCustom[3]["text"] = val
							end
						end,
				},
				fTimelineText8Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 8.03,
					type = "toggle",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[3] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[3] then
								CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] = val
							end
						end,
				},
				fTimelineText8Color = {
					name = "Color",
					desc = "Selects the text color",
					order = 8.04,
					type = "color",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTextCustom[3]["color"]["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextCustom[3]["color"]["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextCustom[3]["color"]["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextCustom[3]["color"]["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["color"] = { r = red, g = green, b = blue, a = alpha }
						end,
				},
				fTimelineText8Font = {
					name = "Font",
					desc = "Selects the font for the timeline and icons",
					order = 8.05,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[3]["font"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["font"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				fTimelineText8Size = {
					name = "Size",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 8.06,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					softMin = 1,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[3]["size"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["size"] = val
						end,
				},
				spacer81 = {
					name = "",
					type = "description",
					order = 8.07,
				},
				fTimelineText8XOffset = {
					name = "x Pos",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 8.08,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					softMin = -100,
					softMax = 100,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[3]["xOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["xOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, val, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				fTimelineText8YOffset = {
					name = "y Pos",
					desc = "Sets the y co-rd for the Timeline frame",
					order = 8.09,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[3]["yOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["yOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, CooldownTimeline.db.profile.fTimelinePosX, val)
						end,
				},
				--[[fTimelineText8Align = {
					name = "Align",
					desc = "X/Y position is relative to this point of the screen",
					order = 8.10,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[3]["align"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["align"] = val
							CooldownTimeline:SetTimelineText()
						end,
				},]]--
				fTimelineText8Anchor = {
					name = "Anchor Point",
					desc = "X/Y position is relative to this point of the screen",
					order = 8.11,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[3]["anchor"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[3]["anchor"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				spacer82 = {
					name = "\n\n",
					type = "description",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[3]["enabled"] end,
					order = 8.12,
				},
				spacer9 = {
					name = "",
					type = "description",
					order = 9.01,
				},
				fTimelineText9Text = {
					name = "Custom Text 4",
					desc = function(info) return CooldownTimeline.GetCustomTagDescription() end,
					type = "input",
					disabled = function(info) return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					order = 9.02,
					width = "double",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[4] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[4]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[4] then
								CooldownTimeline.db.profile.fTimelineTextCustom[4]["text"] = val
							end
						end,
				},
				fTimelineText9Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 9.03,
					type = "toggle",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[4] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[4] then
								CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] = val
							end
						end,
				},
				fTimelineText9Color = {
					name = "Color",
					desc = "Selects the text color",
					order = 9.04,
					type = "color",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTextCustom[4]["color"]["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextCustom[4]["color"]["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextCustom[4]["color"]["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextCustom[4]["color"]["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["color"] = { r = red, g = green, b = blue, a = alpha }
						end,
				},
				fTimelineText9Font = {
					name = "Font",
					desc = "Selects the font for the timeline and icons",
					order = 9.05,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[4]["font"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["font"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				fTimelineText9Size = {
					name = "Size",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 9.06,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					softMin = 1,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[4]["size"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["size"] = val
						end,
				},
				spacer91 = {
					name = "",
					type = "description",
					order = 9.07,
				},
				fTimelineText9XOffset = {
					name = "x Pos",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 9.08,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					softMin = -100,
					softMax = 100,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[4]["xOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["xOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, val, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				fTimelineText9YOffset = {
					name = "y Pos",
					desc = "Sets the y co-rd for the Timeline frame",
					order = 9.09,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[4]["yOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["yOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, CooldownTimeline.db.profile.fTimelinePosX, val)
						end,
				},
				--[[fTimelineText9Align = {
					name = "Align",
					desc = "X/Y position is relative to this point of the screen",
					order = 9.10,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[4]["align"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["align"] = val
							CooldownTimeline:SetTimelineText()
						end,
				},]]--
				fTimelineText9Anchor = {
					name = "Anchor Point",
					desc = "X/Y position is relative to this point of the screen",
					order = 9.11,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[4]["anchor"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[4]["anchor"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				spacer92 = {
					name = "\n\n",
					type = "description",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[4]["enabled"] end,
					order = 9.12,
				},
				spacer10 = {
					name = "",
					type = "description",
					order = 10.01,
				},
				fTimelineText10Text = {
					name = "Custom Text 5",
					desc = function(info) return CooldownTimeline.GetCustomTagDescription() end,
					type = "input",
					disabled = function(info) return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					order = 10.02,
					width = "double",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[5] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[5]["text"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[5] then
								CooldownTimeline.db.profile.fTimelineTextCustom[5]["text"] = val
							end
						end,
				},
				fTimelineText10Enabled = {
					name = "Enabled",
					desc = "Show this text",
					order = 10.03,
					type = "toggle",
					get = function(info)
							if CooldownTimeline.db.profile.fTimelineTextCustom[5] then
								return CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"]
							end
						end,
					set = function(info, val)
							if CooldownTimeline.db.profile.fTimelineTextCustom[5] then
								CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] = val
							end
						end,
				},
				fTimelineText10Color = {
					name = "Color",
					desc = "Selects the text color",
					order = 10.04,
					type = "color",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTextCustom[5]["color"]["r"]
							local g = CooldownTimeline.db.profile.fTimelineTextCustom[5]["color"]["g"]
							local b = CooldownTimeline.db.profile.fTimelineTextCustom[5]["color"]["b"]
							local a = CooldownTimeline.db.profile.fTimelineTextCustom[5]["color"]["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["color"] = { r = red, g = green, b = blue, a = alpha }
						end,
				},
				fTimelineText10Font = {
					name = "Font",
					desc = "Selects the font for the timeline and icons",
					order = 10.05,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					dialogControl = 'LSM30_Font',
					values = AceGUIWidgetLSMlists.font,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[5]["font"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["font"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				fTimelineText10Size = {
					name = "Size",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 10.06,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					softMin = 1,
					softMax = 64,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[5]["size"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["size"] = val
						end,
				},
				spacer101 = {
					name = "",
					type = "description",
					order = 10.07,
				},
				fTimelineText10XOffset = {
					name = "x Pos",
					desc = "Sets the x co-rd for the Timeline frame",
					order = 10.08,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					softMin = -100,
					softMax = 100,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[5]["xOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["xOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, val, CooldownTimeline.db.profile.fTimelinePosY)
						end,
				},
				fTimelineText10YOffset = {
					name = "y Pos",
					desc = "Sets the y co-rd for the Timeline frame",
					order = 10.09,
					type = "range",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					softMin = -50,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[5]["yOffset"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["yOffset"] = val
							--CooldownTimeline.fTimeline:SetPoint(CooldownTimeline.db.profile.fTimelineRelativeTo, CooldownTimeline.db.profile.fTimelinePosX, val)
						end,
				},
				--[[fTimelineText10Align = {
					name = "Align",
					desc = "X/Y position is relative to this point of the screen",
					order = 10.10,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[5]["align"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["align"] = val
							CooldownTimeline:SetTimelineText()
						end,
				},]]--
				fTimelineText10Anchor = {
					name = "Anchor Point",
					desc = "X/Y position is relative to this point of the screen",
					order = 10.11,
					type = "select",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					values = {
							["TOPLEFT"] = "TOPLEFT",
							["TOP"] = "TOP",
							["TOPRIGHT"] = "TOPRIGHT",
							["LEFT"] = "LEFT",
							["CENTER"] = "CENTER",
							["RIGHT"] = "RIGHT",
							["BOTTOMLEFT"] = "BOTTOMLEFT",
							["BOTTOM"] = "BOTTOM",
							["BOTTOMRIGHT"] = "BOTTOMRIGHT",
						},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTextCustom[5]["anchor"] end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTextCustom[5]["anchor"] = val
							--CooldownTimeline:SetTimelineText()
						end,
				},
				spacer102 = {
					name = "\n\n",
					type = "description",
					hidden = function(info)	return not CooldownTimeline.db.profile.fTimelineTextCustom[5]["enabled"] end,
					order = 10.12,
				},
			}
		},
		optionsTracking = {
			name = "Tracking",
			type = "group",
			args = {
				--[[spacer1 = {
					name = "",
					type = "description",
					order = 1.1,
				},]]--
				fTimelineTracking = {
					name = "Timeline Tracking",
					desc = "The timeline will act as a status bar and track whatever is selected",
					order = 1.2,
					type = "select",
					values = {
								["NONE"] = "None",
								["GCD"] = "GCD",
								["ENERGY_TICK"] = "Energy Tick",
								["MANA_TICK"] = "Mana Tick",
								["CLASS_POWER"] = "Class Power",
								["HEALTH"] = "Health",
								["COMBO_POINTS"] = "Combo Points",
								["MH_SWING"] = "Main-hand Swing",
							},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTracking end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTracking = val
						end,
				},
				spacer2 = {
					name = "",
					type = "description",
					order = 2.1,
				},
				fTimelineTrackingInvert = {
					name = "Invert",
					desc = "The bar will appear empty and fill up instead of being full and empty out",
					order = 2.2,
					type = "toggle",
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingInvert end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingInvert = val 
						end,
				},
				fTimelineTrackingReverse = {
					name = "Reverse",
					desc = "Changes the fill direction",
					order = 2.3,
					type = "toggle",
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingReverse end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingReverse = val
							CooldownTimeline.fTimeline:SetReverseFill(val)
						end,
				},
				spacer3 = {
					name = "\n\n\n",
					type = "description",
					order = 3.1,
				},
				fTimelineTrackingSecondary = {
					name = "Secondary Tracking",
					desc = "Tracks a second thing in the form of a spark",
					order = 3.2,
					type = "select",
					values = {
								["NONE"] = "None",
								["GCD"] = "GCD",
								["ENERGY_TICK"] = "Energy Tick",
								["MANA_TICK"] = "Mana Tick",
								["CLASS_POWER"] = "Class Power",
								["HEALTH"] = "Health",
								["COMBO_POINTS"] = "Combo Points",
								["MH_SWING"] = "Main-hand Swing",
							},
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingSecondary end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingSecondary = val
						end,
				},
				spacer4 = {
					name = "",
					type = "description",
					order = 4.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineTrackingInvertSecondary = {
					name = "Invert",
					desc = "The bar will appear empty and fill up instead of being full and empty out",
					order = 4.2,
					type = "toggle",
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingInvertSecondary end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingInvertSecondary = val 
						end,
				},
				fTimelineTrackingReverseSecondary = {
					name = "Reverse",
					desc = "Changes the fill direction",
					order = 4.3,
					type = "toggle",
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingReverseSecondary end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingReverseSecondary = val
						end,
				},
				spacer5 = {
					name = "",
					type = "description",
					order = 5.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineTrackingSecondaryTexture = {
					name = "Spark Texture",
					desc = "Selects the spark texture",
					order = 5.2,
					type = "select",
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
					dialogControl = 'LSM30_Statusbar',
					values = AceGUIWidgetLSMlists.statusbar,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingSecondaryTexture end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingSecondaryTexture = val
							CooldownTimeline.fTimeline.secondaryTracker.bg:SetTexture(SharedMedia:Fetch("statusbar", val))

							local r = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["a"]
							CooldownTimeline.fTimeline.secondaryTracker.bg:SetVertexColor(r, g, b, a)
						end,
				},
				fTimelineTrackingSecondaryTextureColor = {
					name = "Color",
					desc = "Selects the spark color",
					order = 5.3,
					type = "color",
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
					hasAlpha = true,
					get = function(info)
							local r = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["r"]
							local g = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["g"]
							local b = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["b"]
							local a = CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor["a"]
							return r, g, b, a
						end,
					set = function(info, red, green, blue, alpha)
							CooldownTimeline.db.profile.fTimelineTrackingSecondaryTextureColor = { r = red, g = green, b = blue, a = alpha }
							CooldownTimeline.fTimeline.secondaryTracker.bg:SetVertexColor(red, green, blue, alpha)
						end,
				},
				spacer6 = {
					name = "",
					type = "description",
					order = 6.1,
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
				},
				fTimelineTrackingSecondaryWidth = {
					name = "Spark Width",
					desc = "Sets the width for the secondary tracking spark",
					order = 6.2,
					type = "range",
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
					softMin = 0,
					softMax = 20,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingSecondaryWidth end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingSecondaryWidth = val
							CooldownTimeline.fTimeline.secondaryTracker:SetWidth(val)
						end,
				},
				fTimelineTrackingSecondaryHeight = {
					name = "Spark Height",
					desc = "Sets the height for the secondary tracking spark",
					order = 6.3,
					type = "range",
					hidden = function(info)
							if CooldownTimeline.db.profile.fTimelineTrackingSecondary ~= "NONE" then
								return false
							else
								return true
							end
						end,
					softMin = 0,
					softMax = 50,
					get = function(info) return CooldownTimeline.db.profile.fTimelineTrackingSecondaryHeight end,
					set = function(info, val)
							CooldownTimeline.db.profile.fTimelineTrackingSecondaryHeight = val
							CooldownTimeline.fTimeline.secondaryTracker:SetHeight(val)
						end,
				},
			}
		},
	}
}
	
local optionsReady = {
	name = "Ready",
	handler = CooldownTimeline,
	type = 'group',
	args = {
		--[[readyFrameHeader = {
			name = "Ready Frame Options",
			type = "header",
			order = 1
		},]]--
		fReadyIconGrow = {
			name = "Grow direction",
			desc = "Grow Left/Up, Center, or Right/Down",
			order = 1.2,
			type = "select",
			values = { ["LEFT"] = "Left/Up", ["CENTER"] = "Center", ["RIGHT"] = "Right/Down" },
			get = function(info) return CooldownTimeline.db.profile.fReadyIconGrow end,
			set = function(info, val) CooldownTimeline.db.profile.fReadyIconGrow = val end,
		},
		fReadyVertical = {
			name = "Vertical",
			desc = "Changes the orientation from horizontal to vertical",
			type = "toggle",
			order = 1.3,
			set = function(info,val) CooldownTimeline.db.profile.fReadyVertical = val end,
			get = function(info) return CooldownTimeline.db.profile.fReadyVertical end,
		},
		spacer2 = {
			name = "\n\n\n",
			type = "description",
			order = 2.1,
		},
		fReadyX = {
			name = "x Pos",
			desc = "Sets the x co-rd for the Ready frame",
			order = 2.2,
			type = "range",
			softMin = -500,
			softMax = 500,
			get = function(info) return CooldownTimeline.db.profile.fReadyPosX end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyPosX = val
					CooldownTimeline.fReady:SetPoint(CooldownTimeline.db.profile.fReadyRelativeTo, val, CooldownTimeline.db.profile.fReadyPosY)
				end,
		},
		fReadyY = {
			name = "y Pos",
			desc = "Sets the y co-rd for the Ready frame",
			order = 2.3,
			type = "range",
			softMin = -500,
			softMax = 500,
			get = function(info) return CooldownTimeline.db.profile.fReadyPosY end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyPosY = val
					CooldownTimeline.fReady:SetPoint(CooldownTimeline.db.profile.fReadyRelativeTo, CooldownTimeline.db.profile.fReadyPosX, val)
				end,
		},
		spacer3 = {
			name = "",
			type = "description",
			order = 3.1,
		},
		fReadyRelativeTo = {
			name = "Anchor Point",
			desc = "X/Y position is relative to this point of the screen",
			order = 3.2,
			type = "select",
			values = {
				["TOPLEFT"] = "TOPLEFT",
				["TOP"] = "TOP",
				["TOPRIGHT"] = "TOPRIGHT",
				["LEFT"] = "LEFT",
				["CENTER"] = "CENTER",
				["RIGHT"] = "RIGHT",
				["BOTTOMLEFT"] = "BOTTOMLEFT",
				["BOTTOM"] = "BOTTOM",
				["BOTTOMRIGHT"] = "BOTTOMRIGHT",
				},
			get = function(info) return CooldownTimeline.db.profile.fReadyRelativeTo end,
			set = function(info, val)
					CooldownTimeline.fReady:ClearAllPoints()
					
					CooldownTimeline.db.profile.fReadyRelativeTo = val
					
					CooldownTimeline.fReady:SetPoint(val, CooldownTimeline.db.profile.fReadyPosX, CooldownTimeline.db.profile.fReadyPosY)
				end,
		},
		spacer4 = {
			name = "\n\n\n",
			type = "description",
			order = 4.1,
		},
		fReadyTexture = {
			name = "Background Texture",
			desc = "Selects the texture",
			order = 4.2,
			type = "select",
			dialogControl = 'LSM30_Statusbar',
			values = AceGUIWidgetLSMlists.statusbar,
			get = function(info) return CooldownTimeline.db.profile.fReadyTexture end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyTexture = val
					CooldownTimeline.fReady.bg:SetTexture(SharedMedia:Fetch("statusbar", CooldownTimeline.db.profile.fReadyTexture))

					local r = CooldownTimeline.db.profile.fReadyTextureColor["r"]
					local g = CooldownTimeline.db.profile.fReadyTextureColor["g"]
					local b = CooldownTimeline.db.profile.fReadyTextureColor["b"]
					local a = CooldownTimeline.db.profile.fReadyTextureColor["a"]
					CooldownTimeline.fReady.bg:SetVertexColor(r, g, b, a)
				end,
		},
		fReadyTextureColor = {
			name = "Color",
			desc = "Selects the background color",
			order = 4.3,
			type = "color",
			hasAlpha = true,
			get = function(info)
					local r = CooldownTimeline.db.profile.fReadyTextureColor["r"]
					local g = CooldownTimeline.db.profile.fReadyTextureColor["g"]
					local b = CooldownTimeline.db.profile.fReadyTextureColor["b"]
					local a = CooldownTimeline.db.profile.fReadyTextureColor["a"]
					return r, g, b, a
				end,
			set = function(info, red, green, blue, alpha)
					CooldownTimeline.db.profile.fReadyTextureColor = { r = red, g = green, b = blue, a = alpha }
					CooldownTimeline.fReady.bg:SetVertexColor(red, green, blue, alpha)
				end,
		},
		spacer5 = {
			name = "\n\n\n",
			type = "description",
			order = 5.1,
		},
		fReadyIconSize = {
			name = "Icon Size",
			desc = "Sets the size of the icon on the timeline",
			order = 5.2,
			type = "range",
			softMin = 0,
			softMax = 64,
			get = function(info) return CooldownTimeline.db.profile.fReadyIconSize end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyIconSize = val
					for _, child in ipairs(CooldownTimeline.iconTable) do
						if Masque then
							-- Kill masque for this icon(button)
							CooldownTimeline.masqueGroup:RemoveButton(child)
							
							-- Change the settings
							child:SetSize(val, val)
							child:SetPoint("CENTER",0,0)
							child.tex:SetAllPoints(child)
							
							-- Reapply masque
							CooldownTimeline.masqueGroup = Masque:Group("CooldownTimeline")
							CooldownTimeline.masqueGroup:AddButton(child, { Icon = child.tex })	
						else
							child:SetSize(val, val)
							child:SetPoint("CENTER",0,0)
						end
					end
				end,
		},
		fReadyIconPadding = {
			name = "Icon Padding",
			desc = "Sets the padding between icons",
			order = 5.3,
			type = "range",
			softMin = 0,
			softMax = 20,
			get = function(info) return CooldownTimeline.db.profile.fReadyIconPadding end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyIconPadding = val
				end,
		},
		spacer6 = {
			name = "\n\n\n",
			type = "description",
			order = 6.1,
		},
		fReadyIconDuration = {
			name = "Icon Duration",
			desc = "Sets the duration an icon will display as ready",
			order = 6.2,
			type = "range",
			softMin = 0,
			softMax = 10,
			get = function(info) return CooldownTimeline.db.profile.fReadyIconDuration end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyIconDuration = val
				end,
		},
		spacer7 = {
			name = "\n\n\n",
			type = "description",
			order = 7.1,
		},
		fReadyBorder = {
			name = "                    Border Texture",
			desc = "Selects the texture",
			order = 7.2,
			type = "select",
			dialogControl = 'LSM30_Border',
			values = AceGUIWidgetLSMlists.border,
			get = function(info) return CooldownTimeline.db.profile.fReadyBorder end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyBorder = val
					local fReadyBorderSize = CooldownTimeline.db.profile.fReadyBorderSize
					local fReadyBorderInset = CooldownTimeline.db.profile.fReadyBorderInset
					local fReadyBorderPadding = CooldownTimeline.db.profile.fReadyBorderPadding
					local fReadyBorderColor = CooldownTimeline.db.profile.fReadyBorderColor
					
					CooldownTimeline:SetBorder(CooldownTimeline.fReady, val, fReadyBorderSize, fReadyBorderInset)
					CooldownTimeline:SetBorderColor(CooldownTimeline.fReady, fReadyBorderColor)
					CooldownTimeline:SetBorderPoint(CooldownTimeline.fReady, fReadyBorderPadding)
				end,
		},
		fReadyBorderColor = {
			name = "Color",
			desc = "Selects the border color",
			order = 7.3,
			type = "color",
			hasAlpha = true,
			get = function(info)
					local r = CooldownTimeline.db.profile.fReadyBorderColor["r"]
					local g = CooldownTimeline.db.profile.fReadyBorderColor["g"]
					local b = CooldownTimeline.db.profile.fReadyBorderColor["b"]
					local a = CooldownTimeline.db.profile.fReadyBorderColor["a"]
					return r, g, b, a
				end,
			set = function(info, red, green, blue, alpha)
					CooldownTimeline.db.profile.fReadyBorderColor = { r = red, g = green, b = blue, a = alpha }
					CooldownTimeline.fReady.border:SetBackdropBorderColor(red, green, blue, alpha)
				end,
		},
		spacer8 = {
			name = "",
			type = "description",
			order = 8.1,
		},
		fReadyBorderSize = {
			name = "Size",
			desc = "Sets the size of the border",
			order = 8.2,
			type = "range",
			softMin = 1,
			softMax = 40,
			get = function(info) return CooldownTimeline.db.profile.fReadyBorderSize end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyBorderSize = val
					local fReadyBorder = CooldownTimeline.db.profile.fReadyBorder
					local fReadyBorderInset = CooldownTimeline.db.profile.fReadyBorderInset
					local fReadyBorderPadding = CooldownTimeline.db.profile.fReadyBorderPadding
					local fReadyBorderColor = CooldownTimeline.db.profile.fReadyBorderColor
					
					CooldownTimeline:SetBorder(CooldownTimeline.fReady, fReadyBorder, val, fReadyBorderInset)
					CooldownTimeline:SetBorderColor(CooldownTimeline.fReady, fReadyBorderColor)
				end,
		},
		fReadyBorderPadding = {
			name = "Padding",
			desc = "Sets the size of the border",
			order = 8.3,
			type = "range",
			softMin = 0,
			softMax = 40,
			get = function(info) return CooldownTimeline.db.profile.fReadyBorderPadding end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyBorderPadding = val
					local fReadyBorderColor = CooldownTimeline.db.profile.fReadyBorderColor
					
					CooldownTimeline:SetBorderPoint(CooldownTimeline.fReady, val)
					CooldownTimeline:SetBorderColor(CooldownTimeline.fReady, fReadyBorderColor)
				end,
		},
		spacer9 = {
			name = "\n\n\n",
			type = "description",
			order = 9.1,
		},
		fReadyAnimateInType = {
			name = "IN Animation",
			desc = "Select the animation played on show",
			order = 9.2,
			type = "select",
			values = { ["FADE"] = "Fade" },
			get = function(info) return CooldownTimeline.db.profile.fReadyAnimateInType end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyAnimateInType = val
				end,
		},
		fReadyAnimateIn = {
			name = "Enabled",
			desc = "Enable/disable animated on show",
			type = "toggle",
			order = 9.3,
			set = function(info,val) CooldownTimeline.db.profile.fReadyAnimateIn = val end,
			get = function(info) return CooldownTimeline.db.profile.fReadyAnimateIn end,
		},
		spacer10 = {
			name = "",
			type = "description",
			order = 10.1,
		},
		fReadyAnimateOutType = {
			name = "OUT Animation",
			desc = "Select the animation played on hide",
			order = 10.2,
			type = "select",
			values = { ["FADE"] = "Fade" },
			get = function(info) return CooldownTimeline.db.profile.fReadyAnimateOutType end,
			set = function(info, val)
					CooldownTimeline.db.profile.fReadyAnimateOutType = val
				end,
		},
		fReadyAnimateOut = {
			name = "Enabled",
			desc = "Enable/disable animated on hide",
			type = "toggle",
			order = 10.3,
			set = function(info,val) CooldownTimeline.db.profile.fReadyAnimateOut = val end,
			get = function(info) return CooldownTimeline.db.profile.fReadyAnimateOut end,
		},
	}
}

local optionsHighlight = {
	name = "Highlights",
	handler = CooldownTimeline,
	type = 'group',
	args = {
		highlightDescription = {
			name = "Select the cooldowns you which to highlight as importnant \n\n",
			type = "description",
			order = 1.,
		},
		highlightHeadingSpells = {
			name = "Spells",
			type = "header",
			order = 2.1,
		},
		highlight = {
			name = "",
			desc = "",
			order = 2.2,
			type = "multiselect",
			values = function(info)
					CooldownTimeline.characterHighlightList = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelist) do
						local displayName = spell["name"]
						for _, tSpell in pairs(CooldownTimeline.spellTable) do
							if displayName == tSpell["name"] then
								table.insert(CooldownTimeline.characterHighlightList, displayName)
								break
							end
						end
					end

					return CooldownTimeline.characterHighlightList
				end,
			get = function(info, index)
					local displayName = CooldownTimeline.characterHighlightList[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelist) do
						if spell["name"] == displayName then
							return spell["highlight"]
						end
					end
				end,
			set = function(info, index, val)
					local displayName = CooldownTimeline.characterHighlightList[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelist) do
						if spell["name"] == displayName then
							spell["highlight"] = val
							break
						end
					end
				end,
		},
		highlightHeadingItems = {
			name = "Items",
			type = "header",
			order = 3.1,
		},
		highlightItemsDescription1 = {
			name = "\nItems are listed below by the name of the spell cast on use, and many items wont actually generate an item\n",
			type = "description",
			order = 3.2,
		},
		highlightItemsDescription2 = {
			name = "\nYou can remove old and unwanted item spells from the filters options panel\n\n",
			type = "description",
			order = 3.3,
		},
		highlightItems = {
			name = "",
			desc = "",
			order = 3.5,
			type = "multiselect",
			values = function(info)
					CooldownTimeline.characterHighlightItems = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						local displayName = spell["name"]
						for _, tSpell in pairs(CooldownTimeline.spellTable) do
							if displayName == tSpell["name"] then
								table.insert(CooldownTimeline.characterHighlightItems, displayName)
								break
							end
						end
					end

					return CooldownTimeline.characterHighlightItems
				end,
			get = function(info, index)
					local displayName = CooldownTimeline.characterHighlightItems[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						if spell["name"] == displayName then
							return spell["highlight"]
						end
					end
				end,
			set = function(info, index, val)
					local displayName = CooldownTimeline.characterHighlightItems[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						if spell["name"] == displayName then
							spell["highlight"] = val
							break
						end
					end
				end,
		},
		highlightHeadingAuras = {
			name = "Auras",
			type = "header",
			order = 4.1,
		},
		highlightAuras = {
			name = "",
			desc = "",
			order = 4.2,
			type = "multiselect",
			values = function(info)
					local list = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistAuras) do
						local displayName = spell["name"]
						table.insert(list, displayName)
					end
					
					return list
				end,
			get = function(info, index)
					local spell = CooldownTimeline.db.profile.whitelistAuras[index]
					return spell["highlight"]
				end,
			set = function(info, index, val)
					CooldownTimeline.db.profile.whitelistAuras[index]["highlight"] = val
				end,
		},
		highlightHeadingPetSpells = {
			name = "Pet Spells",
			type = "header",
			order = 5.1,
		},
		highlightPet = {
			name = "",
			desc = "",
			order = 5.2,
			type = "multiselect",
			values = function(info)
					CooldownTimeline.characterHighlightPet = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistPet) do
						local displayName = spell["name"]
						for _, tSpell in pairs(CooldownTimeline.spellTable) do
							if displayName == tSpell["name"] then
								table.insert(CooldownTimeline.characterHighlightPet, displayName)
								break
							end
						end
					end

					return CooldownTimeline.characterHighlightPet
				end,
			get = function(info, index)
					local displayName = CooldownTimeline.characterHighlightPet[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistPet) do
						if spell["name"] == displayName then
							return spell["highlight"]
						end
					end
				end,
			set = function(info, index, val)
					local displayName = CooldownTimeline.characterHighlightPet[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistPet) do
						if spell["name"] == displayName then
							spell["highlight"] = val
							break
						end
					end
				end,
		},
	}
}

local optionsFilter = {
	name = "Filters",
	handler = CooldownTimeline,
	type = 'group',
	args = {
		whitelistDescription = {
			name = "Unselect the tickbox to hide the respective icon on the timeline (and ready area)\n\n",
			type = "description",
			order = 1.,
		},
		whitelistHeadingSpells = {
			name = "Spells",
			type = "header",
			order = 2.1,
		},
		whitelist = {
			name = "",
			desc = "",
			order = 2.2,
			type = "multiselect",
			--disabled = true,
			values = function(info)
					CooldownTimeline.characterFilterList = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelist) do
						local displayName = spell["name"]
						for _, tSpell in pairs(CooldownTimeline.spellTable) do
							if displayName == tSpell["name"] then
								table.insert(CooldownTimeline.characterFilterList, displayName)
								break
							end
						end
					end

					return CooldownTimeline.characterFilterList
				end,
			get = function(info, index)
					local displayName = CooldownTimeline.characterFilterList[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelist) do
						if spell["name"] == displayName then
							return spell["tracked"]
						end
					end
				end,
			set = function(info, index, val)
					local displayName = CooldownTimeline.characterFilterList[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelist) do
						if spell["name"] == displayName then
							spell["tracked"] = val
							break
						end
					end
				end,
		},
		whitelistHeadingItems = {
			name = "Items",
			type = "header",
			order = 3.1,
		},
		whitelistItemsDescription1 = {
			name = "\nItems are listed below by the name of the spell cast on use, and many items wont actually generate an item\n",
			type = "description",
			order = 3.2,
		},
		whitelistItemsDescription2 = {
			name = "\nOver time you may gather items that are not longer required(quest items), or should not be listed(recipes).\n\nClicking the 'Clean Item Table' button will remove them - REQUIRES UI RELOAD\n\n",
			type = "description",
			order = 3.3,
		},
		whitelistItemsCleanButton = {
			name = "Clean Item Table",
			desc = "This will remove quest items, and any items clearly not meant to be tracked (such as recipes)\n\nRequires UI Reload",
			order = 3.4,
			type = "execute",
			confirm = true,
			func = function(info)
					CooldownTimeline.db.profile.needToCleanTable = true
					
					for key, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						local _, itemType, itemSubType, _, _, itemClassID, itemSubClassID = GetItemInfoInstant(spell["id"])
						
						if CooldownTimeline:IsValidItemType(itemType) then
							if not CooldownTimeline:IsBlacklisted("ITEM", spell["name"]) then
								table.insert(CooldownTimeline.db.profile.cleanTable, spell)
							end
						end
					end
					
					for key, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						CooldownTimeline.db.profile.whitelistItems[key] = nil
					end
					
					ReloadUI()
				end
		},
		whitelistItems = {
			name = "",
			desc = "",
			order = 3.5,
			type = "multiselect",
			--disabled = true,
			values = function(info)
					CooldownTimeline.characterFilterListItems = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						local displayName = spell["name"]
						for _, tSpell in pairs(CooldownTimeline.spellTable) do
							if displayName == tSpell["name"] then
								table.insert(CooldownTimeline.characterFilterListItems, displayName)
								break
							end
						end
					end

					return CooldownTimeline.characterFilterListItems
				end,
			get = function(info, index)
					local displayName = CooldownTimeline.characterFilterListItems[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						if spell["name"] == displayName then
							return spell["tracked"]
						end
					end
				end,
			set = function(info, index, val)
					local displayName = CooldownTimeline.characterFilterListItems[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistItems) do
						if spell["name"] == displayName then
							spell["tracked"] = val
							break
						end
					end
				end,
		},
		whitelistHeadingAuras = {
			name = "Auras",
			type = "header",
			order = 4.1,
		},
		whitelistAuras = {
			name = "",
			desc = "",
			order = 4.2,
			type = "multiselect",
			--disabled = true,
			values = function(info)
					local list = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistAuras) do
						local displayName = spell["name"]
						table.insert(list, displayName)
					end
					
					return list
				end,
			--values = CooldownTimeline.db.profile.whitelist,
			get = function(info, index)
					local spell = CooldownTimeline.db.profile.whitelistAuras[index]
					return spell["tracked"]
				end,
			set = function(info, index, val)
					CooldownTimeline.db.profile.whitelistAuras[index]["tracked"] = val
				end,
		},
		whitelistHeadingPetSpells = {
			name = "Pet Spells",
			type = "header",
			order = 5.1,
		},
		whitelistPet = {
			name = "",
			desc = "",
			order = 5.2,
			type = "multiselect",
			--disabled = true,
			values = function(info)
					CooldownTimeline.characterFilterListPet = {}
					
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistPet) do
						local displayName = spell["name"]
						for _, tSpell in pairs(CooldownTimeline.spellTable) do
							if displayName == tSpell["name"] then
								table.insert(CooldownTimeline.characterFilterListPet, displayName)
								break
							end
						end
					end

					return CooldownTimeline.characterFilterListPet
				end,
			get = function(info, index)
					local displayName = CooldownTimeline.characterFilterListPet[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistPet) do
						if spell["name"] == displayName then
							return spell["tracked"]
						end
					end
				end,
			set = function(info, index, val)
					local displayName = CooldownTimeline.characterFilterListPet[index]
					for _, spell in pairs(CooldownTimeline.db.profile.whitelistPet) do
						if spell["name"] == displayName then
							spell["tracked"] = val
							break
						end
					end
				end,
		},
	}
}

local defaults = {
    profile = {
		-- Default general values
		debugFrame = false,
		showGCD = true,
		hideOutsideCombat = false,
		onlyShowWhenCoolingDown = false,
		trackItemCooldowns = true,
		trackPetSpells = false,
		longIgnoreThreshold = 600,
		unlockFrames = false,
		stringImportExport = "",
		
		-- Default values for the Cooldown Icons
		fIconSize = 36,
		fIconReadyText = "Ready",
		fIconFont = "",
		fIconFontSize = 13,
		fIconFontColor = { r = 1, g = 1, b = 1, a = 1 },
		fIconGlow = false,
		fIconTextOffset = 0,
		
		fIconReadySound = "None",
		fIconReadyHighlightSound = "None",
		
		fIconBorder = "None",
		fIconBorderColor = { r = 1, g = 1, b = 1, a = 1 },
		fIconBorderSize = 5,
		fIconBorderPadding = 2,
		fIconBorderInset = 0,
		
		fIconHighlightBorder = "",
		fIconHighlightBorderColor = { r = 1, g = 1, b = 1, a = 1 },
		fIconHighlightBorderSize = 5,
		fIconHighlightBorderPadding = 2,
		fIconHighlightBorderInset = 0,

		fIconHighlightPin = false;
		fIconHighlightEffect = "NONE";

		fIconAnimateIn = false,
		fIconAnimateOut = false,
		fIconAnimateInType = "FADE",
		fIconAnimateOutType = "FADE",
	
		-- Default values for the Ready Frame
		fReadyRelativeTo = "CENTER",
		fReadyPosX = 0,
        fReadyPosY = 50,
		fReadyTexture = "",
		fReadyTextureColor = { r = 0.5, g = 0.5, b = 0.5, a = 0.5 },
		fReadyIconSize = 48,
		fReadyIconPadding = 0,
		fReadyIconDuration = 5,
		fReadyIconHighlightDuration = 10,
		fReadyIconGrow = "CENTER",
		fReadyVertical = true,
		
		fReadyBorder = "None",
		fReadyBorderColor = { r = 1, g = 1, b = 1, a = 1 },
		fReadyBorderSize = 5,
		fReadyBorderPadding = 2,
		fReadyBorderInset = 0,
		
		fReadyAnimateIn = false,
		fReadyAnimateOut = false,
		fReadyAnimateInType = "FADE",
		fReadyAnimateOutType = "FADE",
		
		-- Default values for the Timeline Frame
		fTimelineTracking = "NONE",
		fTimelineTrackingReverse = false;
		fTimelineTrackingInvert = false;
		fTimelineMode = "SPLIT",
		fTimelineIconReverseDirection = false;
		fTimelineStack = true,
		fTimelineStackOverlap = true,
		fTimelineStackMaxSize = 36,
		fTimelineIconOffset = 0,
		fTimelineTextOffset = 0,
		
		fTimelineModeAbsLimit = 180,
		
		fTimelineModeSplitCount = 3,
		fTimelineModeSplit1 = 10,
		fTimelineModeSplit2 = 60,
		fTimelineModeSplit3 = 180,
		fTimelineModeSplitLimit = 600,
		
		fTimelineTrackingSecondary = "NONE",
		fTimelineTrackingReverseSecondary = false;
		fTimelineTrackingInvertSecondary = false;
		fTimelineTrackingSecondaryTexture = "",
		fTimelineTrackingSecondaryTextureColor = { r = 1, g = 1, b = 1, a = 1 },
		fTimelineTrackingSecondaryHeight = 50;
		fTimelineTrackingSecondaryWidth = 5;
		
		fTimelineAnimateIn = false,
		fTimelineAnimateOut = false,
		fTimelineAnimateInType = "FADE",
		fTimelineAnimateOutType = "FADE",
		
		fTimelineFont = "",
		fTimelineFontSize = 13,
		fTimelineFontColor = { r = 1, g = 1, b = 1, a = 1 },
		
		fTimelineText = {},
		fTimelineTextCustom = {},
		
		fTimelineRelativeTo = "CENTER",
		fTimelinePosX = 0,
        fTimelinePosY = -100,
        fTimelineWidth = 400,
        fTimelineHeight = 50,
		
		fTimelineBackground = "",
		fTimelineBackgroundColor = { r = 0.5, g = 0.5, b = 0.5, a = 0.5 },
		fTimelineTexture = "",
		fTimelineTextureColor = { r = 0.5, g = 0.5, b = 0.5, a = 0.5 },
		
		fTimelineBorder = "None",
		fTimelineBorderColor = { r = 1, g = 1, b = 1, a = 1 },
		fTimelineBorderSize = 5,
		fTimelineBorderPadding = 2,
		fTimelineBorderInset = 0,
		
		whitelist = {},
		whitelistPet = {},
		whitelistItems = {},
		whitelistAuras = {},
		
		needToCleanTable = false,
		cleanTable = {},
    }
}

function CooldownTimeline:OnInitialize()
    -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("CooldownTimelineDB", defaults, true)
	self.registry = LibStub("AceConfigRegistry-3.0")
	self.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimeline", options)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimelineReady", optionsReady)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimelineTimeline", optionsTimeline)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimelineIcons", optionsIcons)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimelineHighlight", optionsHighlight)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimelineFilter", optionsFilter)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("CooldownTimelineProfiles", self.profile)
	
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimeline", "Cooldown Timeline")
	self.optionsFrame.oIcons = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimelineIcons", "Icons", "Cooldown Timeline")
	self.optionsFrame.oReady = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimelineReady", "Ready", "Cooldown Timeline")
	self.optionsFrame.oTimeline = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimelineTimeline", "Timeline", "Cooldown Timeline")
	self.optionsFrame.oHighlight = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimelineHighlight", "Highlight", "Cooldown Timeline")
	self.optionsFrame.oFilter = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimelineFilter", "Filter", "Cooldown Timeline")
	self.optionsFrame.profile = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CooldownTimelineProfiles", "Profiles", "Cooldown Timeline")
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
    self:RegisterChatCommand("cdtl", "ChatCommand")
    self:RegisterChatCommand("cooldowntimeline", "ChatCommand")
end

function CooldownTimeline:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterEvent("ITEM_LOCK_CHANGED")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	self:RegisterEvent("UNIT_POWER_FREQUENT")
	self:RegisterEvent("UNIT_POWER_UPDATE")
	
	if self.db.profile.debugFrame then
		self:CreateDebugFrame()
	end
	
	self.playerClass, _, _ = UnitClass("player")
	
	-- Create the frames
	self:CreateReadyFrame()
	self:CreateTimelineFrame()
	self:CreateHoldingFrame()
	self:CreateIgnoredFrame()
	
	-- Finish the table clean as needed
	if CooldownTimeline.db.profile.needToCleanTable then
		-- Add all the cleaned values back in
		for _, item in pairs(CooldownTimeline.db.profile.cleanTable) do
			table.insert(CooldownTimeline.db.profile.whitelistItems, item)
		end
		
		-- Clear the cleanTable
		CooldownTimeline.db.profile.cleanTable = {}
		CooldownTimeline.db.profile.needToCleanTable = false
	end
	
	-- Called when the addon is enabled
	self:ScanSpellbook()
	self:ScanInventory()
	self:ScanAuraTable()

	self:ScanCurrentCooldowns()
	
	self:Print("Loaded version: "..version)
	self:Print("Type /cdtl or /cooldowntimeline for options ")
end

function CooldownTimeline:OnDisable()
    -- Called when the addon is disabled
end

function CooldownTimeline:COMBAT_LOG_EVENT_UNFILTERED()
	local _, subevent, _, sourceGUID, _, _, _, destGUID, _, _, _ = CombatLogGetCurrentEventInfo()
	local playerID = UnitGUID("player")
	local petID
	
	local petExists = UnitExists("pet")
	if petExists then
		CooldownTimeline:ScanPetSpellbook()
		petID = UnitGUID("pet")
	end
	
	if sourceGUID == playerID or destGUID == playerID or sourceGUID == petID then
		-- SPELL_CAST_SUCCESS --
		if subevent == "SPELL_CAST_SUCCESS" then
			local _, spellName, _, _, _, _, _, _, _, _, _, _, _ = select(12, CombatLogGetCurrentEventInfo())

			for k, v in pairs(CooldownTimeline.spellTable) do
				if v["name"] == spellName then
					--CooldownTimeline:Print(spellName)
					local matchFound, matchFrame = self:CheckExistingIcons(v["name"])
					
					if matchFound then
						if matchFrame.hasCooldown then
							CooldownTimeline:SendToTimeline(matchFrame)
						end
					else
						if v["type"] == "PETSPELL" then
							local baseDuration, _ = GetSpellBaseCooldown(v["id"])
							local _,_,spellIcon,_,_,_,_ = GetSpellInfo(v["id"])
							
							local data = {}
							data["name"] = spellName
							data["type"] = v["type"]
							data["id"] = v["id"]
							
							-- Stop searching as we have a match
							self:CreateCooldownIcon(data)
							break
						end
						--CooldownTimeline:Print("created CD for "..spellName)
					end
				end
			end
			
		-- SPELL_AURA_APPLIED --
		elseif subevent == "SPELL_AURA_APPLIED" then
			local _, spellName, _, _, _, _, _, _, _, _, _, _, _ = select(12, CombatLogGetCurrentEventInfo())
			
			for _, aura in pairs(CooldownTimeline.aurasToTrack) do
				if spellName == aura["name"] and destGUID == playerID then
					--CooldownTimeline:Print(spellName)
					local matchFound, matchFrame = self:CheckExistingIcons(spellName)
					
					if matchFound then
						if matchFrame.hasCooldown then
							CooldownTimeline:SendToTimeline(matchFrame)
						end
					else
						local data = {}
						data["name"] = aura["name"]
						data["type"] = aura["type"]
						data["id"] = aura["id"]
						
						self:CreateCooldownIcon(data)
					end
					
					break
				end
			end
			
		-- SWING_DAMAGE --
		elseif subevent == "SWING_DAMAGE" then
			local amount, _, _, _, _, _, _, _, _, isOffHand = select(12, CombatLogGetCurrentEventInfo())
			local mhSpeed, ohSpeed = UnitAttackSpeed("player")
			
			if not isOffHand then
				--CooldownTimeline:Print("SWING_DAMAGE - MH - "..tostring(amount))
				CooldownTimeline.fTimeline.mhSwingTime = mhSpeed
				CooldownTimeline.fTimeline.mhSwinging = true
			end
		
		-- SWING_MISSED --
		elseif subevent == "SWING_MISSED" then
			local _, isOffHand, _, _ = select(12, CombatLogGetCurrentEventInfo())
			local mhSpeed, ohSpeed = UnitAttackSpeed("player")
			
			if not isOffHand then
				--CooldownTimeline:Print("SWING_MISSED - MH - "..tostring(amount))
				CooldownTimeline.fTimeline.mhSwingTime = mhSpeed
				CooldownTimeline.fTimeline.mhSwinging = true
			end
		end
	end
end

function CooldownTimeline:PLAYER_REGEN_DISABLED()
	--CooldownTimeline:Print("Combat")
	CooldownTimeline.inCombat = true
end

function CooldownTimeline:PLAYER_REGEN_ENABLED()
	--CooldownTimeline:Print("No Combat")
	CooldownTimeline.inCombat = false
end

function CooldownTimeline:BAG_UPDATE()
	CooldownTimeline:ScanInventory()
	--CooldownTimeline:Print("BAG_UPDATE")
end

function CooldownTimeline:LEARNED_SPELL_IN_TAB()
	CooldownTimeline:ScanSpellbook()
	--CooldownTimeline:Print("LEARNED_SPELL_IN_TAB")
end

function CooldownTimeline:SPELL_UPDATE_COOLDOWN()
	CooldownTimeline.fTimeline.onGCD = true
	--CooldownTimeline:Print("SPELL_UPDATE_COOLDOWN")
end

function CooldownTimeline:UNIT_POWER_FREQUENT(...)
	local _, unitTarget, powerType = ...
	
	if unitTarget == "player" then
		if powerType == "MANA" then
			self.fTimeline.manaMax = UnitPowerMax("player", Enum.PowerType.Mana)
			
			local currentTime = GetTime()
			local currentMana = UnitPower("player", Enum.PowerType.Mana)
			local maxMana = UnitPowerMax("player", Enum.PowerType.Mana)
			local difference = currentMana - self.fTimeline.manaPrevious
			local timeDifference = 0
			
			if self.fTimeline.manaPreviousTickTime then
				timeDifference = currentTime - self.fTimeline.manaPreviousTickTime
			end
			
			-- Start the five second rule
			if difference < 0 then
				self.fTimeline.fiveSecondRule = true
				self.fTimeline.manaTickTime = 6
				self.fTimeline.manaTickInterval = 6
				self.fTimeline.manaPreviousTickTime = currentTime
			end			
			
			-- Check for normal ticks and re-sync
			if difference > 0 then
				local low = 0.1
				local high = 1.9
				
				if self.fTimeline.fiveSecondRule then
					high = 4.9
				end
				
				if timeDifference < low or  timeDifference > high then
					self.fTimeline.manaTickTime = 2
					self.fTimeline.manaTickInterval = 2
					self.fTimeline.manaPreviousTickTime = currentTime
				end
			end
			
			self.fTimeline.manaPrevious = currentMana
		end
	end
end

function CooldownTimeline:UNIT_POWER_UPDATE(...)
	local _, unitTarget, powerType = ...
	--CooldownTimeline:Print("UNIT_POWER_UPDATE - "..GetTime())
	--CooldownTimeline:Print(unitTarget.." - "..powerType)
	
	if unitTarget == "player" then
		if powerType == "ENERGY" then
			self.fTimeline.energyMax = UnitPowerMax("player", Enum.PowerType.Energy)
			
			local current = UnitPower("player", Enum.PowerType.Energy)
			local difference = current - self.fTimeline.energyPrevious
			
			if current < self.fTimeline.energyMax then
				local difference = current - self.fTimeline.energyPrevious
				if (difference > 18 and difference < 22) or (difference > 38 and difference < 42) then
					self.fTimeline.energyLastTickTime = GetTime()
				end
			end
			
			self.fTimeline.energyPrevious = current
		end
	end
end

function CooldownTimeline:ITEM_LOCK_CHANGED(...)
	-- Detect item lock cooldowns
	-- Most commonly trinkets being equipped, that will begin a 30 second cooldown
	local _, bagOrSlotIndex, slotIndex = ...
	
	if slotIndex == nil then
		--CooldownTimeline:Print("Inventory: "..bagOrSlotIndex)
		local itemId = GetInventoryItemID("player", bagOrSlotIndex)
		local itemName = C_Item.GetItemNameByID(itemId)
		local spellName, spellID = GetItemSpell(itemId)
		local start, duration, enabled = GetItemCooldown(itemId)
		
		if duration ~= 0 then
			local matchFound, matchFrame = self:CheckExistingIcons(spellName)
			
			if matchFound then
				if matchFrame.hasCooldown then
					CooldownTimeline:SendToTimeline(matchFrame)
				end
			else
				local data = {}
				data["name"] = spellName
				data["type"] = "ITEM"
				data["id"] = itemId
				
				--CooldownTimeline:Print(itemName..": "..start.." - "..duration)
				self:CreateCooldownIcon(data)
			end
		end
	else
		--CooldownTimeline:Print("Bag/Slot: "..bagOrSlotIndex.."/"..slotIndex)
	end
	
	--CooldownTimeline:Print("ITEM_LOCK_CHANGED")
end

function CooldownTimeline:UNIT_SPELLCAST_SUCCEEDED(...)
	local _, unitTarget, castGUID, spellID = ...
	
	-- Lets find a matched spell in our collection
	if unitTarget == "player" then
		if self.db.profile.debugFrame then
			CooldownTimeline:Print(unitTarget.." - "..castGUID.." - "..spellID)
		end
	
		for _, spell in pairs(CooldownTimeline.spellTable) do
			if spell["type"] == "SPELL" or spell["type"] == "PETSPELL" then
				if spell["id"] == spellID then
					local matchFound, matchFrame = self:CheckExistingIcons(spell["name"])
					
					if matchFound then
						CooldownTimeline:SendToTimeline(matchFrame)
						break
					else
						--local baseDuration, _ = GetSpellBaseCooldown(spellID)
						--local _,_,spellIcon,_,_,_,_ = GetSpellInfo(spellID)
						
						local data = {}
						data["name"] = spell["name"]
						data["type"] = spell["type"]
						data["id"] = spell["id"]
						
						-- Stop searching as we have a match
						self:CreateCooldownIcon(data)
						break
					end
				end
			elseif spell["type"] == "ITEM" then
				--CooldownTimeline:Print("ITEM - "..spell["sid"].." - "..spellID)
				if spell["sid"] == spellID then
					
					local matchFound, matchFrame = self:CheckExistingIcons(spell["name"])
					
					if matchFound then
						-- Some item spell names are common, but had different spell ids
						-- Lets update the icon to the currently used item (eg. different health potions with different icons)
						for _, spell in pairs(CooldownTimeline.spellTable) do
							if spell["type"] == "ITEM" then
								local _, existingSpellID = GetItemSpell(spell["id"])
								
								if spellID == existingSpellID then
									matchFrame.cdID = spell["id"]
									matchFrame.cdIcon = GetItemIcon(matchFrame.cdID)
									matchFrame.tex:SetTexture(matchFrame.cdIcon)
									break
								end
							end
						end
						
						CooldownTimeline:SendToTimeline(matchFrame)
					else
						local data = {}
						data["name"] = spell["name"]
						data["type"] = spell["type"]
						data["id"] = spell["id"]
						
						-- Stop searching as we have a match
						self:CreateCooldownIcon(data)
						break
					end
				end
			end
		end
	end
end

function CooldownTimeline:CreateTestCooldownIcon(name, icon, cooldown)
	local frameName = name.."_"..cooldown
	
	
	local matchFound, matchFrame = self:CheckExistingIcons(frameName)
	if matchFound then
		matchFrame.start = GetTime()
		CooldownTimeline:SendToTimeline(matchFrame)
	else
		local f
		local fIconSize = self.db.profile.fIconSize
		
		if Masque then
			f = CreateFrame("Button", "CooldownTimeline_"..name, CooldownTimeline_Timeline, BackdropTemplateMixin and "BackdropTemplate" or nil)
			f:EnableMouse(false)
			f:SetSize(fIconSize, fIconSize)
			f:SetPoint("CENTER",0,0)
			
			f.tex = f:CreateTexture()
			f.tex:SetAllPoints(f)
			f.tex:SetTexture(132386)
			
			CooldownTimeline.masqueGroup = Masque:Group("CooldownTimeline")
			CooldownTimeline.masqueGroup:AddButton(f, { Icon = f.tex })		
		else
			f = CreateFrame("Frame", "CooldownTimeline_"..name, CooldownTimeline_Timeline, BackdropTemplateMixin and "BackdropTemplate" or nil)
			f:SetPoint("CENTER", self.db.profile.fTimelineWidth, 0)
			f:SetSize(fIconSize, fIconSize)
		
			f.tex = f:CreateTexture()
			f.tex:SetAllPoints(f)
			f.tex:SetTexture(icon)
		end
		
		f.cdUniqueID = CooldownTimeline:AssignUniqueID()
		if self.db.profile.debugFrame then
			f.textID = f:CreateFontString(nil,"ARTWORK")
			f.textID:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
			f.textID:SetPoint("TOPLEFT",2,-2)
			f.textID:SetText(f.cdUniqueID)
		end
		
		f.hasCooldown = true
		f.updateCount = 0
		f.readyStart = 0
		
		f.text = f:CreateFontString(nil,"ARTWORK")
		f.text:SetFont(SharedMedia:Fetch("font", self.db.profile.fIconFont), self.db.profile.fIconFontSize, "OUTLINE")
		f.text:SetPoint("CENTER",0,0)
		f.text:SetText("0.0")
		
		f.iconIsHolding = false
		f.readyTimerRunning = false
		f.readyTimerStart = 0
		
		f.start = GetTime()
		f.duration = cooldown
		f.cdRemaining = 0
		
		f.yOffset = 0
		
		-- Set the icon border
		local fIconBorderInset = self.db.profile.fIconBorderInset
		local fIconBorderPadding = self.db.profile.fIconBorderPadding
		
		f.border = CreateFrame("Frame", frameName.."_Border",f , BackdropTemplateMixin and "BackdropTemplate" or nil)
		f.border:SetParent(f)
		f.border:SetPoint("TOPLEFT", f, "TOPLEFT", -fIconBorderPadding, fIconBorderPadding)
		f.border:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", fIconBorderPadding, -fIconBorderPadding)
		
		f.border:SetBackdrop({
			bgFile = SharedMedia:Fetch("background", "None"),
			edgeFile = SharedMedia:Fetch("border", self.db.profile.fIconBorder),
			tile = false,
			tileSize = 0,
			edgeSize = CooldownTimeline.db.profile.fIconBorderSize,
			insets = { left = fIconBorderInset, right = fIconBorderInset, top = fIconBorderInset, bottom = fIconBorderInset }
		})
		
		r = CooldownTimeline.db.profile.fIconBorderColor["r"]
		g = CooldownTimeline.db.profile.fIconBorderColor["g"]
		b = CooldownTimeline.db.profile.fIconBorderColor["b"]
		a = CooldownTimeline.db.profile.fIconBorderColor["a"]
		f.border:SetBackdropBorderColor(r, g, b, a)
		
		-- Set the icon hightlight border
		local fIconHighlightBorderInset = self.db.profile.fIconHighlightBorderInset
		local fIconHighlightBorderPadding = self.db.profile.fIconHighlightBorderPadding
		
		f.highlightBorder = CreateFrame("Frame", frameName.."_HL_Border", f, BackdropTemplateMixin and "BackdropTemplate" or nil)
		f.highlightBorder:SetParent(f)
		f.highlightBorder:SetPoint("TOPLEFT", f, "TOPLEFT", -fIconHighlightBorderPadding, fIconHighlightBorderPadding)
		f.highlightBorder:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", fIconHighlightBorderPadding, -fIconHighlightBorderPadding)
		
		f.highlightBorder:SetBackdrop({
			bgFile = SharedMedia:Fetch("background", "None"),
			edgeFile = SharedMedia:Fetch("border", self.db.profile.fIconHighlightBorder),
			tile = false,
			tileSize = 0,
			edgeSize = CooldownTimeline.db.profile.fIconHighlightBorderSize,
			insets = { left = fIconHighlightBorderInset, right = fIconHighlightBorderInset, top = fIconHighlightBorderInset, bottom = fIconHighlightBorderInset }
		})
		
		r = CooldownTimeline.db.profile.fIconHighlightBorderColor["r"]
		g = CooldownTimeline.db.profile.fIconHighlightBorderColor["g"]
		b = CooldownTimeline.db.profile.fIconHighlightBorderColor["b"]
		a = CooldownTimeline.db.profile.fIconHighlightBorderColor["a"]
		f.highlightBorder:SetBackdropBorderColor(r, g, b, a)
		
		f.highlightBorder:SetAlpha(0)
		
		f:HookScript("OnUpdate", function(self,elapsed)
			f.readyTimerDuration = CooldownTimeline.db.profile.fReadyIconDuration
			if not f.iconIsHolding then
				self.cdRemaining = (f.start + f.duration) - GetTime()
				local timeLeftDisplay = CooldownTimeline:ConvertToReadableTime(self.cdRemaining)
				
				if self.cdRemaining >= 0 then
					if f:GetParent():GetName() == "CooldownTimeline_Timeline" then
						f.text:SetText(timeLeftDisplay)
						
						local mode = CooldownTimeline.db.profile.fTimelineMode
						local position
						if mode == "LINEAR" then
							position = CooldownTimeline:CalcLinearPercentPosition(self.cdRemaining, cooldown)
						elseif mode == "LINEAR_ABS" then
							position = CooldownTimeline:CalcLinearAbsolutePosition(self.cdRemaining, nil)
						elseif mode == "SPLIT" then
							position = CooldownTimeline:CalcSplitAbsolutePosition(self.cdRemaining)
						end
						
						local iconRelativeTo = "RIGHT"
				
						if CooldownTimeline.db.profile.fTimelineIconReverseDirection then
							iconRelativeTo = "LEFT"
						end
						
						f:SetPoint(iconRelativeTo, position - CooldownTimeline.db.profile.fTimelineWidth, self.yOffset)
					end
				elseif f.readyTimerRunning then
					if (f.readyTimerStart + f.readyTimerDuration) - GetTime() < 0 then
						CooldownTimeline:SendToHolding(f)
					end
				else
					CooldownTimeline:SendToReady(f)
					f.readyTimerRunning = true
					f.readyTimerStart = GetTime()
				end
			end
		end)
		
		table.insert(CooldownTimeline.iconTable, f)
	end
end

function CooldownTimeline:CreateIgnoredFrame()
	self.fIgnored = CreateFrame("Frame", "CooldownTimeline_Ignored", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	
	self.fIgnored:SetPoint("CENTER", UIParent, "CENTER", 0, 315)
	self.fIgnored:SetWidth(50)
	self.fIgnored:SetHeight(50)

	self.fIgnored:SetBackdrop({
		bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		edgeFile = nil, tile = true, tileSize = 32, edgeSize = 0, 
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	
	self.fIgnored:HookScript("OnUpdate", function(self,elapsed)
		local children = { CooldownTimeline.fIgnored:GetChildren() }
		
		local childCount = 0
		for _, child in ipairs(children) do
			-- A positive value will grow to the right
			local position = CooldownTimeline.db.profile.fReadyIconSize * childCount
			
			child:ClearAllPoints()
			child:SetPoint("CENTER",  position, 0)
			childCount = childCount + 1
		end
	end)
	
	if not self.db.profile.debugFrame then
		self.fIgnored:Hide()
	end
	
	self.fIgnored.text = self.fIgnored:CreateFontString(nil,"ARTWORK")
	self.fIgnored.text:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
	self.fIgnored.text:SetPoint("TOPLEFT",0,15)
	self.fIgnored.text:SetText("Ignored Frame")
end

function CooldownTimeline:CreateHoldingFrame()
	self.fHolding = CreateFrame("Frame", "CooldownTimeline_Holding", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	self.fHolding:SetPoint("CENTER", 0, 400)
	self.fHolding:SetSize(50, 50)

	self.fHolding:SetBackdrop({
		bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		edgeFile = nil, tile = true, tileSize = 32, edgeSize = 0, 
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	
	self.fHolding:HookScript("OnUpdate", function(self,elapsed)
		local children = { CooldownTimeline.fHolding:GetChildren() }
		
		local childCount = 0
		for _, child in ipairs(children) do
			-- A positive value will grow to the right
			local position = CooldownTimeline.db.profile.fReadyIconSize * childCount
			
			child:SetPoint("CENTER",  position, 0)
			childCount = childCount + 1
		end
	end)
	
	if not self.db.profile.debugFrame then
		self.fHolding:Hide()
	end
	
	self.fHolding.text = self.fHolding:CreateFontString(nil,"ARTWORK")
	self.fHolding.text:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
	self.fHolding.text:SetPoint("TOPLEFT",0,15)
	self.fHolding.text:SetText("Holding Frame")
end

function CooldownTimeline:CalcLinearPercentPosition(timeLeft, cooldown)
	local percent = ( timeLeft / cooldown ) * 100
	
	local fIconSize = CooldownTimeline.db.profile.fIconSize
	
	local timelineWidthAdjusted = CooldownTimeline.db.profile.fTimelineWidth - fIconSize
	
	local position = ( timelineWidthAdjusted / 100 ) * percent
	position = position + fIconSize
	
	return position
end

function CooldownTimeline:CalcLinearAbsolutePosition(timeLeft, id)
	local absolute = CooldownTimeline.db.profile.fTimelineModeAbsLimit
	
	local fIconSize = CooldownTimeline.db.profile.fIconSize
	
	local timelineWidthAdjusted = CooldownTimeline.db.profile.fTimelineWidth - fIconSize
	
	local percent = ( timeLeft / absolute ) * 100
	local position = ( timelineWidthAdjusted / 100 ) * percent
	position = position + fIconSize
	
	if timeLeft > absolute then
		position = timelineWidthAdjusted + fIconSize
	end
	
	return position
end

function CooldownTimeline:CalcSplitAbsolutePosition(timeLeft, id)
	local fTimelineModeSplitCount = CooldownTimeline.db.profile.fTimelineModeSplitCount
	local splitTime1 = CooldownTimeline.db.profile.fTimelineModeSplit1
	local splitTime2 = CooldownTimeline.db.profile.fTimelineModeSplit2
	local splitTime3 = CooldownTimeline.db.profile.fTimelineModeSplit3
	local absolute = CooldownTimeline.db.profile.fTimelineModeSplitLimit
	
	local fIconSize = CooldownTimeline.db.profile.fIconSize
	local timelineWidthAdjusted = CooldownTimeline.db.profile.fTimelineWidth - fIconSize
	
	local position = 0
	local regionSize = timelineWidthAdjusted / (fTimelineModeSplitCount + 1)
	local regionAdjustment = 0
	local percent = 0
	
	if timeLeft > absolute then
		position = CooldownTimeline.db.profile.fTimelineWidth
	else
		if fTimelineModeSplitCount == 1 then
			if timeLeft > splitTime1 then
				regionAdjustment = regionSize
				
				local adjustedTimeLeft = timeLeft - splitTime1
				percent = adjustedTimeLeft / (absolute - splitTime1)
			else
				percent = timeLeft / splitTime1
			end
		elseif fTimelineModeSplitCount == 2 then
			if timeLeft > splitTime2 then
				regionAdjustment = regionSize * 2
				
				local adjustedTimeLeft = timeLeft - splitTime2
				percent = adjustedTimeLeft / (absolute - splitTime2)
			elseif timeLeft > splitTime1 then
				regionAdjustment = regionSize
				
				local adjustedTimeLeft = timeLeft - splitTime1
				percent = adjustedTimeLeft / (splitTime2 - splitTime1)
			else
				percent = timeLeft / splitTime1
			end
		elseif fTimelineModeSplitCount == 3 then
			if timeLeft > splitTime3 then
				regionAdjustment = regionSize * 3
				
				local adjustedTimeLeft = timeLeft - splitTime3
				percent = adjustedTimeLeft / (absolute - splitTime3)
			elseif timeLeft > splitTime2 then
				regionAdjustment = regionSize * 2
				
				local adjustedTimeLeft = timeLeft - splitTime2
				percent = adjustedTimeLeft / (splitTime3 - splitTime2)
			elseif timeLeft > splitTime1 then
				regionAdjustment = regionSize
				
				local adjustedTimeLeft = timeLeft - splitTime1
				percent = adjustedTimeLeft / (splitTime2 - splitTime1)
			else
				percent = timeLeft / splitTime1
			end
		end
		
		position = position + regionAdjustment
		position = position + (regionSize * percent)
		position = position + fIconSize
	end
	
	return position
end

function CooldownTimeline:CalcLogAbsolutePosition(timeLeft, id)
	local position = 0
	
	-- y = alogx where a = 50
	
	for x = 0, 10, 1 do
		local a = 25
		local y = math.log(x) * a
	end
	
	return position
end

function CooldownTimeline:CalcCurveLinearPosition(timeLeft, cooldown)
	local position = 0
	
	local percent = ( timeLeft / (cooldown / 1000)) * 100
	
	local percentCalc = math.sqrt(10000 - ((100 -percent) * (100 -percent)))
	
	percent = 100 - percentCalc
	
	position = ( CooldownTimeline.db.profile.fTimelineWidth / 100 ) * -percent
	position = position + CooldownTimeline.db.profile.fTimelineWidth
	position = position + (CooldownTimeline.db.profile.fIconSize / 2)
	
	return position
end

function CooldownTimeline:CreateDebugFrame()
	self.fDebug = CreateFrame("Frame", "CooldownTimeline_Debug", UIParent, BackdropTemplateMixin and "BackdropTemplate" or nil)
	self.fDebug:SetPoint("CENTER", 400, 200)
	self.fDebug:SetSize(400, 200)

	self.fDebug:SetBackdrop({
		bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
		edgeFile = nil, tile = true, tileSize = 32, edgeSize = 0, 
		insets = { left = 0, right = 0, top = 0, bottom = 0 }
	})
	
	self.fDebug.text = self.fDebug:CreateFontString(nil,"ARTWORK")
	self.fDebug.text:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
	self.fDebug.text:SetPoint("TOPLEFT",0,15)
	self.fDebug.text:SetText("Debug Frame")
	
	self.fDebug.text1 = self.fDebug:CreateFontString(nil,"ARTWORK")
	self.fDebug.text1:SetFont("Fonts\\ARIALN.ttf", 10, "OUTLINE")
	self.fDebug.text1:SetPoint("RIGHT",0,0)
	self.fDebug.text1:SetText(self.db.profile.fTimelinePosX.." - "..self.db.profile.fTimelinePosY)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_Reload", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Reload")
	b:SetPoint("TOPLEFT", 10, -10)
	b:SetScript("OnClick", function()
		ReloadUI()
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_Options", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Options")
	b:SetPoint("TOPLEFT", 10, -35)
	b:SetScript("OnClick", function()
		InterfaceOptionsFrame_OpenToCategory(CooldownTimeline.optionsFrame)
        InterfaceOptionsFrame_OpenToCategory(CooldownTimeline.optionsFrame.oTimeline)
		InterfaceOptionsFrame_OpenToCategory(CooldownTimeline.optionsFrame)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_RunTest", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Run Test1")
	b:SetPoint("TOPLEFT", 10, -60)
	b:SetScript("OnClick", function()
		CooldownTimeline:TestCode1()
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_RunTest", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Run Test2")
	b:SetPoint("TOPLEFT", 10, -85)
	b:SetScript("OnClick", function()
		CooldownTimeline:TestCode2()
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon1", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 5")
	b:SetPoint("TOPLEFT", 100, -10)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 5)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon2", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 10")
	b:SetPoint("TOPLEFT", 100, -35)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 10)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon3", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 30")
	b:SetPoint("TOPLEFT", 100, -60)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 30)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon4", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 60")
	b:SetPoint("TOPLEFT", 100, -85)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 60)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon5", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 70")
	b:SetPoint("TOPLEFT", 100, -110)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 70)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon6", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 180")
	b:SetPoint("TOPLEFT", 100, -135)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 180)
	end)
	
	local b = CreateFrame("Button", "CooldownTimeline_Debug_TestIcon7", CooldownTimeline_Debug, "UIPanelButtonTemplate", BackdropTemplateMixin and "BackdropTemplate" or nil)
	b:SetSize(80 ,25) -- width, height
	b:SetText("Test Icon 360")
	b:SetPoint("TOPLEFT", 100, -160)
	b:SetScript("OnClick", function()
		CooldownTimeline:CreateTestCooldownIcon("Test", 1, 360)
	end)
end

function CooldownTimeline:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(CooldownTimeline.optionsFrame)
        InterfaceOptionsFrame_OpenToCategory(CooldownTimeline.optionsFrame.oTimeline)
        InterfaceOptionsFrame_OpenToCategory(CooldownTimeline.optionsFrame)
		--LibStub("AceConfigDialog-3.0"):Open("CooldownTimeline")
    else
        --LibStub("AceConfigCmd-3.0"):HandleCommand("wh", "WelcomeHome", input)
    end
end